@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,13 +16,15 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #include "nzbget.h"
 #include "Connection.h"
 #include "Log.h"
+#include "FileSystem.h"
+#include "Options.h"
 
 static const int CONNECTION_READBUFFER_SIZE = 1024;
 #ifndef HAVE_GETADDRINFO
@@ -30,16 +33,12 @@ std::unique_ptr<Mutex> Connection::m_getHostByNameMutex;
 #endif
 #endif
 
-class ConnectionFinalizer
-{
-public:
-	~ConnectionFinalizer()
-	{
-		Connection::Final();
-	}
-};
-
-std::unique_ptr<ConnectionFinalizer> m_connectionFinalizer;
+#if defined(__linux__) && !defined(__ANDROID__)
+// Activate DNS resolving workaround for Android:
+//  - this is only necessary in general Linux build if we want it to run on Android.
+//  - the workaround isn't needed when targeting specifically Android using Android NDK.
+#define ANDROID_RESOLVE
+#endif
 
 void closesocket_gracefully(SOCKET socket)
 {
@@ -79,6 +78,10 @@ void closesocket_gracefully(SOCKET socket)
 	closesocket(socket);
 }
 
+#ifdef ANDROID_RESOLVE
+CString ResolveAndroidHost(const char* host);
+#endif
+
 void Connection::Init()
 {
 	debug("Initializing global connection data");
@@ -104,8 +107,6 @@ void Connection::Init()
 	m_getHostByNameMutex = std::make_unique<Mutex>();
 #endif
 #endif
-
-	m_connectionFinalizer = std::make_unique<ConnectionFinalizer>();
 }
 
 void Connection::Final()
@@ -121,6 +122,9 @@ Connection::Connection(const char* host, int port, bool tls) :
 	debug("Creating Connection");
 
 	m_readBuf.Reserve(CONNECTION_READBUFFER_SIZE + 1);
+#ifndef DISABLE_TLS
+	m_certVerifLevel = Options::ECertVerifLevel::cvStrict;
+#endif
 }
 
 Connection::Connection(SOCKET socket, bool tls)
@@ -139,6 +143,7 @@ Connection::Connection(SOCKET socket, bool tls)
 #ifndef DISABLE_TLS
 	m_tlsSocket = nullptr;
 	m_tlsError = false;
+	m_certVerifLevel = Options::ECertVerifLevel::cvStrict;
 #endif
 }
 
@@ -146,7 +151,7 @@ Connection::~Connection()
 {
 	debug("Destroying Connection");
 
-	Disconnect();
+	Connection::Disconnect();
 }
 
 void Connection::SetSuppressErrors(bool suppressErrors)
@@ -210,6 +215,8 @@ bool Connection::Bind()
 		return true;
 	}
 
+	int errcode = 0;
+
 #ifndef WIN32
 	if (m_host && m_host[0] == '/')
 	{
@@ -246,7 +253,7 @@ bool Connection::Bind()
 
 		memset(&addr_hints, 0, sizeof(addr_hints));
 		addr_hints.ai_family = m_ipVersion == ipV4 ? AF_INET : m_ipVersion == ipV6 ? AF_INET6 : AF_UNSPEC;
-		addr_hints.ai_socktype = SOCK_STREAM,
+		addr_hints.ai_socktype = SOCK_STREAM;
 		addr_hints.ai_flags = AI_PASSIVE;    // For wildcard IP address
 
 		BString<100> portStr("%d", m_port);
@@ -263,7 +270,6 @@ bool Connection::Bind()
 			return false;
 		}
 
-		m_broken = false;
 		m_socket = INVALID_SOCKET;
 		for (addr = addr_list; addr != nullptr; addr = addr->ai_next)
 		{
@@ -282,6 +288,7 @@ bool Connection::Bind()
 					break;
 				}
 				// Connection failed
+				errcode = GetLastNetworkError();
 				closesocket(m_socket);
 				m_socket = INVALID_SOCKET;
 			}
@@ -322,6 +329,7 @@ bool Connection::Bind()
 		if (res == -1)
 		{
 			// Connection failed
+			errcode = GetLastNetworkError();
 			closesocket(m_socket);
 			m_socket = INVALID_SOCKET;
 		}
@@ -330,7 +338,7 @@ bool Connection::Bind()
 
 	if (m_socket == INVALID_SOCKET)
 	{
-		ReportError("Binding socket failed for %s", m_host, true);
+		ReportError("Binding socket failed for %s", m_host, true, errcode);
 		return false;
 	}
 
@@ -357,7 +365,7 @@ int Connection::WriteLine(const char* buffer)
 	int res = send(m_socket, buffer, strlen(buffer), 0);
 	if (res <= 0)
 	{
-		m_broken = true;
+		m_status = csBroken;
 	}
 
 	return res;
@@ -378,7 +386,7 @@ bool Connection::Send(const char* buffer, int size)
 		int res = send(m_socket, buffer + bytesSent, size-bytesSent, 0);
 		if (res <= 0)
 		{
-			m_broken = true;
+			m_status = csBroken;
 			return false;
 		}
 		bytesSent += res;
@@ -407,13 +415,17 @@ char* Connection::ReadLine(char* buffer, int size, int* bytesReadOut)
 			if (bufAvail < 0)
 			{
 				ReportError("Could not receive data on socket from %s", m_host, true);
-				m_broken = true;
+				m_status = csBroken;
 				break;
 			}
 			else if (bufAvail == 0)
 			{
 				break;
 			}
+			else
+			{
+				m_totalBytesRead += bufAvail;
+			}
 			bufPtr = m_readBuf;
 			m_readBuf[bufAvail] = '\0';
 		}
@@ -456,8 +468,6 @@ char* Connection::ReadLine(char* buffer, int size, int* bytesReadOut)
 		*bytesReadOut = bytesRead;
 	}
 
-	m_totalBytesRead += bytesRead;
-
 	if (inpBuffer == buffer)
 	{
 		return nullptr;
@@ -492,7 +502,7 @@ std::unique_ptr<Connection> Connection::Accept()
 
 int Connection::TryRecv(char* buffer, int size)
 {
-	debug("Receiving data");
+	//debug("Receiving data");
 
 	memset(buffer, 0, size);
 
@@ -502,13 +512,17 @@ int Connection::TryRecv(char* buffer, int size)
 	{
 		ReportError("Could not receive data on socket from %s", m_host, true);
 	}
+	else
+	{
+		m_totalBytesRead += received;
+	}
 
 	return received;
 }
 
 bool Connection::Recv(char * buffer, int size)
 {
-	debug("Receiving data (full buffer)");
+	//debug("Receiving data (full buffer)");
 
 	memset(buffer, 0, size);
 
@@ -537,6 +551,7 @@ bool Connection::Recv(char * buffer, int size)
 		}
 		bufPtr += received;
 		NeedBytes -= received;
+		m_totalBytesRead += received;
 	}
 	return true;
 }
@@ -546,7 +561,6 @@ bool Connection::DoConnect()
 	debug("Do connecting");
 
 	m_socket = INVALID_SOCKET;
-	m_broken = false;
 
 #ifndef WIN32
 	if (m_host && m_host[0] == '/')
@@ -588,6 +602,19 @@ bool Connection::DoConnect()
 		BString<100> portStr("%d", m_port);
 
 		int res = getaddrinfo(m_host, portStr, &addr_hints, &addr_list);
+		debug("getaddrinfo for %s: %i", *m_host, res);
+
+#ifdef ANDROID_RESOLVE
+		if (res != 0)
+		{
+			CString resolvedHost = ResolveAndroidHost(m_host);
+			if (!resolvedHost.Empty())
+			{
+				res = getaddrinfo(resolvedHost, portStr, &addr_hints, &addr_list);
+			}
+		}
+#endif
+
 		if (res != 0)
 		{
 			ReportError("Could not resolve hostname %s", m_host, true
@@ -689,7 +716,7 @@ bool Connection::DoConnect()
 	}
 
 #ifndef DISABLE_TLS
-	if (m_tls && !StartTls(true, nullptr, nullptr))
+	if (m_tls && !StartTls(true, "", ""))
 	{
 		return false;
 	}
@@ -768,18 +795,15 @@ bool Connection::ConnectWithTimeout(void* address, int address_len)
 	ret = connect(m_socket, (struct sockaddr*)address, address_len);
 	if (ret < 0)
 	{
+		int err = GetLastNetworkError();
 #ifdef WIN32
-		int err = WSAGetLastError();
 		if (err != WSAEWOULDBLOCK)
-		{
-			return false;
-		}
 #else
-		if (errno != EINPROGRESS)
+		if (err != EINPROGRESS)
+#endif
 		{
 			return false;
 		}
-#endif
 	}
 
 	//connect succeeded right away?
@@ -883,10 +907,9 @@ void Connection::Cancel()
 	debug("Cancelling connection");
 	if (m_socket != INVALID_SOCKET)
 	{
-		EStatus status = m_status;
 		m_status = csCancelled;
 
-		if (status == csListening)
+		if (m_forceClose)
 		{
 			SOCKET socket = m_socket;
 			m_socket = INVALID_SOCKET;
@@ -900,7 +923,16 @@ void Connection::Cancel()
 	}
 }
 
-void Connection::ReportError(const char* msgPrefix, const char* msgArg, bool PrintErrCode, int herrno, const char* herrMsg)
+int Connection::GetLastNetworkError()
+{
+#ifdef WIN32
+	return WSAGetLastError();
+#else
+	return errno;
+#endif
+}
+
+void Connection::ReportError(const char* msgPrefix, const char* msgArg, bool printErrCode, int errCode, const char* errMsg)
 {
 #ifndef DISABLE_TLS
 	if (m_tlsError)
@@ -913,34 +945,34 @@ void Connection::ReportError(const char* msgPrefix, const char* msgArg, bool Pri
 
 	BString<1024> errPrefix(msgPrefix, msgArg);
 
-	if (PrintErrCode)
+	if (printErrCode)
 	{
-#ifdef WIN32
-		int ErrCode = WSAGetLastError();
-		char errMsg[1024];
-		errMsg[0] = '\0';
-		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, ErrCode, 0, errMsg, 1024, nullptr);
-		errMsg[1024-1] = '\0';
-#else
-		const char* errMsg = herrMsg;
-		int ErrCode = herrno;
-		if (herrno == 0)
+		BString<1024> printErrMsg;
+		if (errCode == 0)
 		{
-			ErrCode = errno;
-			errMsg = strerror(ErrCode);
+			errCode = GetLastNetworkError();
 		}
-		else if (!herrMsg)
+		if (errMsg)
 		{
-			errMsg = hstrerror(ErrCode);
-		}
-#endif
-		if (m_suppressErrors)
-		{
-			debug("%s: ErrNo %i, %s", *errPrefix, ErrCode, errMsg);
+			printErrMsg = errMsg;
 		}
 		else
 		{
-			PrintError(BString<1024>("%s: ErrNo %i, %s", *errPrefix, ErrCode, errMsg));
+#ifdef WIN32
+			FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, errCode, 0, printErrMsg, printErrMsg.Capacity(), nullptr);
+			printErrMsg[1024-1] = '\0';
+#else
+			printErrMsg = strerror(errCode);
+#endif
+		}
+
+		if (m_suppressErrors)
+		{
+			debug("%s: Error %i - %s", *errPrefix, errCode, (const char*)printErrMsg);
+		}
+		else
+		{
+			PrintError(BString<1024>("%s: Error %i - %s", *errPrefix, errCode, (const char*)printErrMsg));
 		}
 	}
 	else
@@ -966,7 +998,7 @@ bool Connection::StartTls(bool isClient, const char* certFile, const char* keyFi
 {
 	debug("Starting TLS");
 
-	m_tlsSocket = std::make_unique<ConTlsSocket>(m_socket, isClient, m_host, certFile, keyFile, m_cipher, this);
+	m_tlsSocket = std::make_unique<ConTlsSocket>(m_socket, isClient, m_host, certFile, keyFile, m_cipher, m_certVerifLevel, this);
 	m_tlsSocket->SetSuppressErrors(m_suppressErrors);
 
 	return m_tlsSocket->Start();
@@ -1058,7 +1090,7 @@ in_addr_t Connection::ResolveHostAddr(const char* host)
 #endif
 		if (err)
 		{
-			ReportError("Could not resolve hostname %s", host, true, h_errnop);
+			ReportError("Could not resolve hostname %s", host, true, h_errnop, hstrerror(h_errnop));
 			return INADDR_NONE;
 		}
 
@@ -1091,7 +1123,10 @@ const char* Connection::GetRemoteAddr()
 			inet_ntop_t* inet_ntop = (inet_ntop_t*)GetProcAddress(module, "inet_ntop");
 			if (inet_ntop)
 			{
-				inet_ntop(((sockaddr_in*)&peerName)->sin_family, &((sockaddr_in*)&peerName)->sin_addr,
+				inet_ntop(((sockaddr_in*)&peerName)->sin_family,
+					((sockaddr_in*)&peerName)->sin_family == AF_INET6 ?
+					(void*)&((sockaddr_in6*)&peerName)->sin6_addr :
+					(void*)&((sockaddr_in*)&peerName)->sin_addr,
 					m_remoteAddr, m_remoteAddr.Capacity());
 			}
 			FreeLibrary(module);
@@ -1101,7 +1136,10 @@ const char* Connection::GetRemoteAddr()
 			m_remoteAddr = inet_ntoa(((sockaddr_in*)&peerName)->sin_addr);
 		}
 #else
-		inet_ntop(((sockaddr_in*)&peerName)->sin_family, &((sockaddr_in*)&peerName)->sin_addr,
+		inet_ntop(((sockaddr_in*)&peerName)->sin_family,
+			((sockaddr_in*)&peerName)->sin_family == AF_INET6 ?
+			(void*)&((sockaddr_in6*)&peerName)->sin6_addr :
+			(void*)&((sockaddr_in*)&peerName)->sin_addr,
 			m_remoteAddr, m_remoteAddr.Capacity());
 #endif
 		m_remoteAddr[m_remoteAddr.Capacity() - 1] = '\0';
@@ -1116,3 +1154,210 @@ int Connection::FetchTotalBytesRead()
 	m_totalBytesRead = 0;
 	return total;
 }
+
+
+#ifdef ANDROID_RESOLVE
+
+//******************************************************************************
+// Android resolver proxy from AOSP (reworked):
+// https://github.com/aosp-mirror/platform_bionic/blob/6c1d23f059986e4ee6f52c44a4944089aae9181d/libc/dns/net/gethnamaddr.c
+
+#define	MAXALIASES	35
+#define	MAXADDRS	35
+#define ALIGNBYTES (sizeof(uintptr_t) - 1)
+#define ALIGN(p) (((uintptr_t)(p) + ALIGNBYTES) &~ ALIGNBYTES)
+
+// This should be synchronized to ResponseCode.h
+static const int DnsProxyQueryResult = 222;
+
+FILE* android_open_proxy()
+{
+	const char* cache_mode = getenv("ANDROID_DNS_MODE");
+	bool use_proxy = (cache_mode == nullptr || strcmp(cache_mode, "local") != 0);
+	if (!use_proxy) {
+		return nullptr;
+	}
+
+	int s = socket(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0);
+	if (s == -1) {
+		return nullptr;
+	}
+
+	const int one = 1;
+	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
+
+	struct sockaddr_un proxy_addr;
+	memset(&proxy_addr, 0, sizeof(proxy_addr));
+	proxy_addr.sun_family = AF_UNIX;
+	strncpy(proxy_addr.sun_path, "/dev/socket/dnsproxyd", sizeof(proxy_addr.sun_path));
+
+	if (connect(s, (const struct sockaddr*) &proxy_addr, sizeof(proxy_addr)) != 0) {
+		close(s);
+		return nullptr;
+	}
+
+	return fdopen(s, "r+");
+}
+
+static struct hostent * android_read_hostent(FILE* proxy, struct hostent* hp, char* hbuf, size_t hbuflen)
+{
+	uint32_t size;
+	char buf[4];
+	if (fread(buf, 1, sizeof(buf), proxy) != sizeof(buf)) return nullptr;
+
+	// This is reading serialized data from system/netd/server/DnsProxyListener.cpp
+	// and changes here need to be matched there.
+	int result_code = strtol(buf, nullptr, 10);
+	if (result_code != DnsProxyQueryResult) {
+		fread(&size, 1, sizeof(size), proxy);
+		return nullptr;
+	}
+
+	if (fread(&size, 1, sizeof(size), proxy) != sizeof(size)) return nullptr;
+	size = ntohl(size);
+
+	memset(hp, 0, sizeof(*hp));
+	char *ptr = hbuf;
+	char *hbuf_end = hbuf + hbuflen;
+
+	if (ptr + size > hbuf_end) {
+		return nullptr;
+	}
+	if (fread(ptr, 1, size, proxy) != size) return nullptr;
+	hp->h_name = ptr;
+	ptr += size;
+
+	char *aliases_ptrs[MAXALIASES];
+	char **aliases = &aliases_ptrs[0];
+
+	while (1) {
+		if (fread(&size, 1, sizeof(size), proxy) != sizeof(size)) return nullptr;
+		size = ntohl(size);
+
+		if (size == 0) {
+			*aliases = nullptr;
+			break;
+		}
+		if (ptr + size > hbuf_end) {
+			return nullptr;
+		}
+		if (fread(ptr, 1, size, proxy) != size) return nullptr;
+		if (aliases < &aliases_ptrs[MAXALIASES - 1]) {
+		  *aliases++ = ptr;
+		}
+		ptr += size;
+	}
+
+	// Fix alignment after variable-length data.
+	ptr = (char*)ALIGN(ptr);
+
+	int aliases_len = ((int)(aliases - aliases_ptrs) + 1) * sizeof(*hp->h_aliases);
+	if (ptr + aliases_len > hbuf_end) {
+		return nullptr;
+	}
+	hp->h_aliases = (char**)ptr;
+	memcpy(ptr, aliases_ptrs, aliases_len);
+	ptr += aliases_len;
+
+	if (fread(&size, 1, sizeof(size), proxy) != sizeof(size)) return nullptr;
+	hp->h_addrtype = ntohl(size);
+
+	if (fread(&size, 1, sizeof(size), proxy) != sizeof(size)) return nullptr;
+	hp->h_length = ntohl(size);
+
+	char *addr_ptrs[MAXADDRS];
+	char **addr_p = &addr_ptrs[0];
+
+	while (1) {
+		if (fread(&size, 1, sizeof(size), proxy) != sizeof(size)) return nullptr;
+		size = ntohl(size);
+		if (size == 0) {
+			*addr_p = nullptr;
+			break;
+		}
+		if (ptr + size > hbuf_end) {
+			return nullptr;
+		}
+		if (fread(ptr, 1, size, proxy) != size) return nullptr;
+		if (addr_p < &addr_ptrs[MAXADDRS - 1]) {
+		  *addr_p++ = ptr;
+		}
+		ptr += size;
+	}
+
+	// Fix alignment after variable-length data.
+	ptr = (char*)ALIGN(ptr);
+
+	int addrs_len = ((int)(addr_p - addr_ptrs) + 1) * sizeof(*hp->h_addr_list);
+	if (ptr + addrs_len > hbuf_end) {
+		return nullptr;
+	}
+	hp->h_addr_list = (char**)ptr;
+	memcpy(ptr, addr_ptrs, addrs_len);
+	return hp;
+}
+
+// very similar in proxy-ness to android_getaddrinfo_proxy
+static struct hostent * android_gethostbyname_internal(const char *name, int af,
+	struct hostent *hp, char *hbuf, size_t hbuflen)
+{
+	FILE* proxy = android_open_proxy();
+	if (proxy == nullptr) {
+		return nullptr;
+	}
+
+	// This is writing to system/netd/server/DnsProxyListener.cpp and changes
+	// here need to be matched there.
+	if (fprintf(proxy, "gethostbyname %s %s %d",
+			"^",
+			name == nullptr ? "^" : name,
+			af) < 0) {
+		fclose(proxy);
+		return nullptr;
+	}
+
+	if (fputc(0, proxy) == EOF || fflush(proxy) != 0) {
+		fclose(proxy);
+		return nullptr;
+	}
+
+	struct hostent* result = android_read_hostent(proxy, hp, hbuf, hbuflen);
+	fclose(proxy);
+	return result;
+}
+
+CString ResolveAndroidHost(const char* host)
+{
+	debug("ResolveAndroidHost");
+
+	bool android = FileSystem::DirectoryExists("/data/dalvik-cache");
+	if (!android)
+	{
+		debug("Not android");
+		return nullptr;
+	}
+
+	struct hostent hinfobuf;
+	char strbuf[1024];
+
+	struct hostent* hinfo = android_gethostbyname_internal(host, AF_INET, &hinfobuf, strbuf, sizeof(strbuf));
+	if (hinfo == nullptr)
+	{
+		// trying IPv6
+		hinfo = android_gethostbyname_internal(host, AF_INET6, &hinfobuf, strbuf, sizeof(strbuf));
+	}
+
+	if (hinfo == nullptr)
+	{
+		debug("android_gethostbyname_r failed");
+		return nullptr;
+	}
+
+	BString<1024> result;
+	inet_ntop(hinfo->h_addrtype, hinfo->h_addr_list[0], result, result.Capacity());
+
+	debug("android_gethostbyname_r returned %s", *result);
+	return *result;
+}
+
+#endif
