@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2008-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef TLSSOCKET_H
@@ -22,18 +23,31 @@
 
 #ifndef DISABLE_TLS
 
-#include "NString.h"
-
 class TlsSocket
 {
 public:
-	TlsSocket(SOCKET socket, bool isClient, const char* host,
-		const char* certFile, const char* keyFile, const char* cipher) :
-		m_socket(socket), m_isClient(isClient), m_host(host),
-		m_certFile(certFile), m_keyFile(keyFile), m_cipher(cipher) {}
+	TlsSocket(
+		SOCKET socket,
+		bool isClient,
+		const char* host,
+		const char* certFile,
+		const char* keyFile,
+		const char* cipher,
+		int certVerifLevel)
+		: m_socket(socket)
+		, m_isClient(isClient)
+		, m_host(host ? host : "")
+		, m_certFile(certFile ? certFile : "")
+		, m_keyFile(keyFile ? keyFile : "")
+		, m_cipher(cipher ? cipher : "")
+		, m_certVerifLevel(certVerifLevel) {}
+
 	virtual ~TlsSocket();
+
 	static void Init();
-	static void InitOptions(const char* certStore) { m_certStore = certStore; }
+	static void InitOptions(const char* certStore);
+	static void Final();
+
 	bool Start();
 	void Close();
 	int Send(const char* buffer, int size);
@@ -44,27 +58,32 @@ protected:
 	virtual void PrintError(const char* errMsg);
 
 private:
-	bool m_isClient;
-	CString m_host;
-	CString m_certFile;
-	CString m_keyFile;
-	CString m_cipher;
+	static void InitX509Store(const std::string& certStore);
+	static void FreeX509Store(X509_STORE* store);
+	static X509_STORE* m_X509Store;
+	static std::string m_certStore;
+
 	SOCKET m_socket;
+
+	std::string m_host;
+	std::string m_certFile;
+	std::string m_keyFile;
+	std::string m_cipher;
+
+	bool m_isClient;
 	bool m_suppressErrors = false;
 	bool m_initialized = false;
 	bool m_connected = false;
-	int m_retCode;
-	static CString m_certStore;
 
-	// using "void*" to prevent the including of GnuTLS/OpenSSL header files into TlsSocket.h
+	int m_retCode;
+	int m_certVerifLevel;
+
+	// using "void*" to prevent the including of OpenSSL header files into TlsSocket.h
 	void* m_context = nullptr;
 	void* m_session = nullptr;
 
 	void ReportError(const char* errMsg, bool suppressable = true);
 	bool ValidateCert();
-
-	static void Final();
-	friend class TlsSocketFinalizer;
 };
 
 #endif
