@@ -1,5 +1,5 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2013-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
@@ -23,16 +23,20 @@
 #include "Util.h"
 
 FeedInfo::FeedInfo(int id, const char* name, const char* url, bool backlog, int interval,
-		const char* filter, bool pauseNzb, const char* category, int priority, const char* feedScript) :
+		const char* filter, bool pauseNzb, const char* category, int priority, const char* extensions) :
 	m_backlog(backlog), m_interval(interval), m_pauseNzb(pauseNzb), m_priority(priority)
 {
 	m_id = id;
 	m_name = name ? name : "";
+	if (m_name.Length() == 0)
+	{
+		m_name.Format("Feed%i", m_id);
+	}
 	m_url = url ? url : "";
 	m_filter = filter ? filter : "";
 	m_filterHash = Util::HashBJ96(m_filter, strlen(m_filter), 0);
 	m_category = category ? category : "";
-	m_feedScript = feedScript ? feedScript : "";
+	m_extensions = extensions ? extensions : "";
 }
 
 
