@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -98,6 +99,13 @@ public:
 		nfArticle,
 		nfNzb
 	};
+	enum ECertVerifLevel
+	{
+		cvNone,
+		cvMinimal,
+		cvStrict,
+		Count
+	};
 
 	class OptEntry
 	{
@@ -169,12 +177,15 @@ public:
 		virtual void AddNewsServer(int id, bool active, const char* name, const char* host,
 			int port, int ipVersion, const char* user, const char* pass, bool joinGroup,
 			bool tls, const char* cipher, int maxConnections, int retention,
-			int level, int group, bool optional) = 0;
-		virtual void AddFeed(int id, const char* name, const char* url, int interval,
-			const char* filter, bool backlog, bool pauseNzb, const char* category,
-			int priority, const char* extensions) {}
-		virtual void AddTask(int id, int hours, int minutes, int weekDaysBits, ESchedulerCommand command,
-			const char* param) {}
+			int level, int group, bool optional, unsigned int certVerificationfLevel) = 0;
+		virtual void AddFeed([[maybe_unused]] int id, [[maybe_unused]] const char* name,
+			[[maybe_unused]] const char* url, [[maybe_unused]] int interval,
+			[[maybe_unused]] const char* filter, [[maybe_unused]] bool backlog,
+			[[maybe_unused]] bool pauseNzb, [[maybe_unused]] const char* category,
+			[[maybe_unused]] int priority, [[maybe_unused]] const char* extensions) {}
+		virtual void AddTask([[maybe_unused]] int id, [[maybe_unused]] int hours, [[maybe_unused]] int minutes,
+			[[maybe_unused]] int weekDaysBits, [[maybe_unused]] ESchedulerCommand command,
+			[[maybe_unused]] const char* param) {}
 		virtual void SetupFirstStart() {}
 	};
 
@@ -191,7 +202,7 @@ public:
 		ESchedulerCommand command, const char* param);
 
 	// Options
-	const char* GetConfigFilename() { return m_configFilename; }
+	const char* GetConfigFilename() const { return m_configFilename; }
 	bool GetConfigErrors() { return m_configErrors; }
 	const char* GetAppDir() { return m_appDir; }
 	const char* GetDestDir() { return m_destDir; }
@@ -201,9 +212,8 @@ public:
 	const char* GetNzbDir() { return m_nzbDir; }
 	const char* GetWebDir() { return m_webDir; }
 	const char* GetConfigTemplate() { return m_configTemplate; }
-	const char* GetScriptDir() { return m_scriptDir; }
+	const char* GetScriptDir() const { return m_scriptDir; }
 	const char* GetRequiredDir() { return m_requiredDir; }
-	bool GetBrokenLog() const { return m_brokenLog; }
 	bool GetNzbLog() const { return m_nzbLog; }
 	EMessageTarget GetInfoTarget() const { return m_infoTarget; }
 	EMessageTarget GetWarningTarget() const { return m_warningTarget; }
@@ -211,17 +221,17 @@ public:
 	EMessageTarget GetDebugTarget() const { return m_debugTarget; }
 	EMessageTarget GetDetailTarget() const { return m_detailTarget; }
 	int GetArticleTimeout() { return m_articleTimeout; }
+	int GetArticleReadChunkSize() { return m_articleReadChunkSize; }
 	int GetUrlTimeout() { return m_urlTimeout; }
-	int GetTerminateTimeout() { return m_terminateTimeout; }
 	int GetRemoteTimeout() { return m_remoteTimeout; }
-	bool GetDecode() { return m_decode; };
+	bool GetRawArticle() { return m_rawArticle; };
+	bool GetSkipWrite() { return m_skipWrite; };
 	bool GetAppendCategoryDir() { return m_appendCategoryDir; }
 	bool GetContinuePartial() { return m_continuePartial; }
 	int GetArticleRetries() { return m_articleRetries; }
 	int GetArticleInterval() { return m_articleInterval; }
 	int GetUrlRetries() { return m_urlRetries; }
 	int GetUrlInterval() { return m_urlInterval; }
-	bool GetSaveQueue() { return m_saveQueue; }
 	bool GetFlushQueue() { return m_flushQueue; }
 	bool GetDupeCheck() { return m_dupeCheck; }
 	const char* GetControlIp() { return m_controlIp; }
@@ -243,9 +253,8 @@ public:
 	const char* GetLockFile() { return m_lockFile; }
 	const char* GetDaemonUsername() { return m_daemonUsername; }
 	EOutputMode GetOutputMode() { return m_outputMode; }
-	bool GetReloadQueue() { return m_reloadQueue; }
 	int GetUrlConnections() { return m_urlConnections; }
-	int GetLogBufferSize() { return m_logBufferSize; }
+	int GetLogBuffer() { return m_logBuffer; }
 	EWriteLog GetWriteLog() { return m_writeLog; }
 	const char* GetLogFile() { return m_logFile; }
 	int GetRotateLog() { return m_rotateLog; }
@@ -259,8 +268,8 @@ public:
 	int GetParThreads() { return m_parThreads; }
 	bool GetRarRename() { return m_rarRename; }
 	EHealthCheck GetHealthCheck() { return m_healthCheck; }
-	const char* GetScriptOrder() { return m_scriptOrder; }
-	const char* GetExtensions() { return m_extensions; }
+	const char* GetScriptOrder() const { return m_scriptOrder; }
+	const char* GetExtensions() const { return m_extensions; }
 	int GetUMask() { return m_umask; }
 	int GetUpdateInterval() {return m_updateInterval; }
 	bool GetCursesNzbName() { return m_cursesNzbName; }
@@ -280,7 +289,6 @@ public:
 	bool GetNzbCleanupDisk() { return m_nzbCleanupDisk; }
 	int GetParTimeLimit() { return m_parTimeLimit; }
 	int GetKeepHistory() { return m_keepHistory; }
-	bool GetAccurateRate() { return m_accurateRate; }
 	bool GetUnpack() { return m_unpack; }
 	bool GetDirectUnpack() { return m_directUnpack; }
 	bool GetUnpackCleanupDisk() { return m_unpackCleanupDisk; }
@@ -290,6 +298,8 @@ public:
 	bool GetUnpackPauseQueue() { return m_unpackPauseQueue; }
 	const char* GetExtCleanupDisk() { return m_extCleanupDisk; }
 	const char* GetParIgnoreExt() { return m_parIgnoreExt; }
+	const char* GetRenameIgnoreExt() { return m_renameIgnoreExt; }
+	bool GetRenameAfterUnpack() { return m_renameAfterUnpack; }
 	const char* GetUnpackIgnoreExt() { return m_unpackIgnoreExt; }
 	int GetFeedHistory() { return m_feedHistory; }
 	bool GetUrlForce() { return m_urlForce; }
@@ -297,13 +307,14 @@ public:
 	int GetPropagationDelay() { return m_propagationDelay; }
 	int GetArticleCache() { return m_articleCache; }
 	int GetEventInterval() { return m_eventInterval; }
-	const char* GetShellOverride() { return m_shellOverride; }
+	const std::string& GetShellOverride() { return m_shellOverride; }
 	int GetMonthlyQuota() { return m_monthlyQuota; }
 	int GetQuotaStartDay() { return m_quotaStartDay; }
 	int GetDailyQuota() { return m_dailyQuota; }
 	bool GetDirectRename() { return m_directRename; }
 	bool GetReorderFiles() { return m_reorderFiles; }
 	EFileNaming GetFileNaming() { return m_fileNaming; }
+	int GetDownloadRate() const { return m_downloadRate; }
 
 	Categories* GetCategories() { return &m_categories; }
 	Category* FindCategory(const char* name, bool searchAliases) { return m_categories.FindCategory(name, searchAliases); }
@@ -315,26 +326,10 @@ public:
 	bool GetDaemonMode() { return m_daemonMode; }
 	void SetRemoteClientMode(bool remoteClientMode) { m_remoteClientMode = remoteClientMode; }
 	bool GetRemoteClientMode() { return m_remoteClientMode; }
-	void SetPauseDownload(bool pauseDownload) { m_pauseDownload = pauseDownload; }
-	bool GetPauseDownload() const { return m_pauseDownload; }
-	void SetPausePostProcess(bool pausePostProcess) { m_pausePostProcess = pausePostProcess; }
-	bool GetPausePostProcess() const { return m_pausePostProcess; }
-	void SetPauseScan(bool pauseScan) { m_pauseScan = pauseScan; }
-	bool GetPauseScan() const { return m_pauseScan; }
-	void SetTempPauseDownload(bool tempPauseDownload) { m_tempPauseDownload = tempPauseDownload; }
-	bool GetTempPauseDownload() const { return m_tempPauseDownload; }
-	bool GetTempPausePostprocess() const { return m_tempPausePostprocess; }
-	void SetTempPausePostprocess(bool tempPausePostprocess) { m_tempPausePostprocess = tempPausePostprocess; }
-	void SetDownloadRate(int rate) { m_downloadRate = rate; }
-	int GetDownloadRate() const { return m_downloadRate; }
-	void SetResumeTime(time_t resumeTime) { m_resumeTime = resumeTime; }
-	time_t GetResumeTime() const { return m_resumeTime; }
-	void SetLocalTimeOffset(int localTimeOffset) { m_localTimeOffset = localTimeOffset; }
-	int GetLocalTimeOffset() { return m_localTimeOffset; }
-	void SetQuotaReached(bool quotaReached) { m_quotaReached = quotaReached; }
-	bool GetQuotaReached() { return m_quotaReached; }
 
 private:
+	void CheckDirs();
+
 	OptEntries m_optEntries;
 	Mutex m_optEntriesMutex;
 	Categories m_categories;
@@ -362,12 +357,12 @@ private:
 	EMessageTarget m_errorTarget = mtScreen;
 	EMessageTarget m_debugTarget = mtNone;
 	EMessageTarget m_detailTarget = mtScreen;
-	bool m_decode = true;
-	bool m_brokenLog = false;
+	bool m_skipWrite = false;
+	bool m_rawArticle = false;
 	bool m_nzbLog = false;
 	int m_articleTimeout = 0;
+	int m_articleReadChunkSize = 4;
 	int m_urlTimeout = 0;
-	int m_terminateTimeout = 0;
 	int m_remoteTimeout = 0;
 	bool m_appendCategoryDir = false;
 	bool m_continuePartial = false;
@@ -375,7 +370,6 @@ private:
 	int m_articleInterval = 0;
 	int m_urlRetries = 0;
 	int m_urlInterval = 0;
-	bool m_saveQueue = false;
 	bool m_flushQueue = false;
 	bool m_dupeCheck = false;
 	CString m_controlIp;
@@ -397,9 +391,8 @@ private:
 	CString m_lockFile;
 	CString m_daemonUsername;
 	EOutputMode m_outputMode = omLoggable;
-	bool m_reloadQueue = false;
 	int m_urlConnections = 0;
-	int m_logBufferSize = 0;
+	int m_logBuffer = 0;
 	EWriteLog m_writeLog = wlAppend;
 	int m_rotateLog = 0;
 	CString m_logFile;
@@ -435,7 +428,6 @@ private:
 	bool m_nzbCleanupDisk = false;
 	int m_parTimeLimit = 0;
 	int m_keepHistory = 0;
-	bool m_accurateRate = false;
 	bool m_unpack = false;
 	bool m_directUnpack = false;
 	bool m_unpackCleanupDisk = false;
@@ -452,26 +444,20 @@ private:
 	int m_propagationDelay = 0;
 	int m_articleCache = 0;
 	int m_eventInterval = 0;
-	CString m_shellOverride;
+	std::string m_shellOverride;
 	int m_monthlyQuota = 0;
 	int m_quotaStartDay = 0;
 	int m_dailyQuota = 0;
 	bool m_reorderFiles = false;
 	EFileNaming m_fileNaming = nfArticle;
+	bool m_renameAfterUnpack = true;
+	CString m_renameIgnoreExt;
+	int m_downloadRate = 0;
 
-	// Current state
+	// Application mode
 	bool m_serverMode = false;
 	bool m_daemonMode = false;
 	bool m_remoteClientMode = false;
-	bool m_pauseDownload = false;
-	bool m_pausePostProcess = false;
-	bool m_pauseScan = false;
-	bool m_tempPauseDownload = true;
-	bool m_tempPausePostprocess = true;
-	int m_downloadRate = 0;
-	time_t m_resumeTime = 0;
-	int m_localTimeOffset = 0;
-	bool m_quotaReached = false;
 
 	void Init(const char* exeName, const char* configFilename, bool noConfig,
 		CmdOptList* commandLineOptions, bool noDiskAccess, Extender* extender);
