@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -24,6 +25,7 @@
 #include "Log.h"
 #include "NzbFile.h"
 #include "Options.h"
+#include "WorkState.h"
 #include "CommandLineParser.h"
 #include "ScriptConfig.h"
 #include "Thread.h"
@@ -53,24 +55,33 @@
 #include "StackTrace.h"
 #include "CommandScript.h"
 #include "YEncode.h"
+#include "ExtensionManager.h"
+#include "SystemInfo.h"
+
 #ifdef WIN32
 #include "WinService.h"
 #include "WinConsole.h"
 #include "WebDownloader.h"
-#endif
-#ifdef ENABLE_TESTS
-#include "TestMain.h"
+#include "Utf8.h"
 #endif
 #ifndef DISABLE_NSERV
 #include "NServMain.h"
 #endif
 
+#ifdef DEBUG
+#include <sstream>
+#include <iostream>
+#endif
+
+#include <locale>
+
 // Prototypes
 void RunMain();
 
 // Globals
 Log* g_Log;
 Options* g_Options;
+WorkState* g_WorkState;
 ServerPool* g_ServerPool;
 QueueCoordinator* g_QueueCoordinator;
 UrlCoordinator* g_UrlCoordinator;
@@ -86,15 +97,18 @@ ArticleCache* g_ArticleCache;
 QueueScriptCoordinator* g_QueueScriptCoordinator;
 ServiceCoordinator* g_ServiceCoordinator;
 ScriptConfig* g_ScriptConfig;
-CommandScriptLog* g_CommandScriptLog; 
+CommandScriptLog* g_CommandScriptLog;
+ExtensionManager::Manager* g_ExtensionManager;
+System::SystemInfo* g_SystemInfo;
+
 #ifdef WIN32
 WinConsole* g_WinConsole;
 #endif
+
 int g_ArgumentCount;
 char* (*g_EnvironmentVariables)[] = nullptr;
 char* (*g_Arguments)[] = nullptr;
 
-
 /*
  * Main entry point
  */
@@ -110,8 +124,12 @@ int main(int argc, char *argv[], char *argp[])
 #endif
 		);
 #endif
+
+	SetConsoleOutputCP(CP_UTF8);
 #endif
 
+	setlocale(LC_CTYPE, "");
+
 	Util::Init();
 	YEncode::init();
 
@@ -119,20 +137,6 @@ int main(int argc, char *argv[], char *argp[])
 	g_Arguments = (char*(*)[])argv;
 	g_EnvironmentVariables = (char*(*)[])argp;
 
-	if (argc > 1 && (!strcmp(argv[1], "-tests") || !strcmp(argv[1], "--tests")))
-	{
-#ifdef ENABLE_TESTS
-		return TestMain(argc, argv);
-#else
-		printf("ERROR: Could not start tests, the program was compiled without tests\n");
-		return 1;
-#endif
-	}
-
-#ifdef ENABLE_TESTS
-	TestCleanup();
-#endif
-
 	if (argc > 1 && (!strcmp(argv[1], "--nserv")))
 	{
 #ifndef DISABLE_NSERV
@@ -179,7 +183,7 @@ public:
 	virtual void AddNewsServer(int id, bool active, const char* name, const char* host,
 		int port, int ipVersion, const char* user, const char* pass, bool joinGroup,
 		bool tls, const char* cipher, int maxConnections, int retention,
-		int level, int group, bool optional);
+		int level, int group, bool optional, unsigned int certVerificationfLevel);
 	virtual void AddFeed(int id, const char* name, const char* url, int interval,
 		const char* filter, bool backlog, bool pauseNzb, const char* category,
 		int priority, const char* feedScript);
@@ -193,6 +197,7 @@ private:
 	// globals
 	std::unique_ptr<Log> m_log;
 	std::unique_ptr<Options> m_options;
+	std::unique_ptr<WorkState> m_workState;
 	std::unique_ptr<ServerPool> m_serverPool;
 	std::unique_ptr<QueueCoordinator> m_queueCoordinator;
 	std::unique_ptr<UrlCoordinator> m_urlCoordinator;
@@ -209,6 +214,9 @@ private:
 	std::unique_ptr<ServiceCoordinator> m_serviceCoordinator;
 	std::unique_ptr<ScriptConfig> m_scriptConfig;
 	std::unique_ptr<CommandScriptLog> m_commandScriptLog;
+	std::unique_ptr<ExtensionManager::Manager> m_extensionManager;
+	std::unique_ptr<System::SystemInfo> m_systemInfo;
+
 #ifdef WIN32
 	std::unique_ptr<WinConsole> m_winConsole;
 #endif
@@ -223,13 +231,17 @@ private:
 
 	bool m_reloading = false;
 	bool m_daemonized = false;
+	bool m_stopped = false;
+	Mutex m_waitMutex;
+	ConditionVar m_waitCond;
 
 	void Init();
+	void Final();
 	void BootConfig();
 	void CreateGlobals();
 	void Cleanup();
 	void PrintOptions();
-	bool ProcessDirect();
+	void ProcessDirect();
 	void ProcessClientRequest();
 	void ProcessWebGet();
 	void ProcessSigVerify();
@@ -275,17 +287,29 @@ void NZBGet::Init()
 	BootConfig();
 
 #ifndef WIN32
-	if (m_options->GetUMask() < 01000)
+	mode_t uMask = static_cast<mode_t>(m_options->GetUMask());
+	if (uMask < 01000)
 	{
 		/* set newly created file permissions */
-		umask(m_options->GetUMask());
+		FileSystem::uMask = uMask;
+		umask(FileSystem::uMask);
 	}
+	else
+	{
+		FileSystem::uMask = umask(0);
+		umask(FileSystem::uMask);
+	}
+
+#ifdef DEBUG
+	debug("Using %o umask", FileSystem::uMask);
+#endif
+
 #endif
 
 	m_scanner->InitOptions();
 	m_queueScriptCoordinator->InitOptions();
 #ifndef DISABLE_TLS
-	TlsSocket::InitOptions(g_Options->GetCertCheck() ? g_Options->GetCertStore() : nullptr);
+	TlsSocket::InitOptions(g_Options->GetCertCheck() ? g_Options->GetCertStore() : "");
 #endif
 
 	if (m_commandLineParser->GetDaemonMode())
@@ -309,6 +333,19 @@ void NZBGet::Init()
 		info("nzbget %s remote-mode", Util::VersionRevision());
 	}
 
+	info("using %s", m_options->GetConfigFilename());
+	info("nzbget runs on %s:%i", m_options->GetControlIp(), m_options->GetControlPort());
+
+#ifdef DEBUG
+	std::stringstream ss;
+	ss << *m_systemInfo;
+	std::string line;
+	while (std::getline(ss, line))
+	{
+		detail("%s", line.c_str());
+	}
+#endif
+
 	m_reloading = false;
 
 	if (!m_commandLineParser->GetRemoteClientMode())
@@ -320,6 +357,17 @@ void NZBGet::Init()
 	InstallErrorHandler();
 }
 
+void NZBGet::Final()
+{
+	if (!m_reloading)
+	{
+#ifndef DISABLE_TLS
+		TlsSocket::Final();
+#endif
+		Connection::Final();
+	}
+}
+
 void NZBGet::CreateGlobals()
 {
 #ifdef WIN32
@@ -327,6 +375,9 @@ void NZBGet::CreateGlobals()
 	g_WinConsole = m_winConsole.get();
 #endif
 
+	m_workState = std::make_unique<WorkState>();
+	g_WorkState = m_workState.get();
+
 	m_serviceCoordinator = std::make_unique<ServiceCoordinator>();
 	g_ServiceCoordinator = m_serviceCoordinator.get();
 
@@ -375,6 +426,12 @@ void NZBGet::CreateGlobals()
 	m_commandScriptLog = std::make_unique<CommandScriptLog>();
 	g_CommandScriptLog = m_commandScriptLog.get();
 
+	m_extensionManager = std::make_unique<ExtensionManager::Manager>();
+	g_ExtensionManager = m_extensionManager.get();
+
+	m_systemInfo = std::make_unique<System::SystemInfo>();
+	g_SystemInfo = m_systemInfo.get();
+
 	m_scheduler = std::make_unique<Scheduler>();
 
 	m_diskService = std::make_unique<DiskService>();
@@ -400,7 +457,8 @@ void NZBGet::BootConfig()
 		m_commandLineParser->GetNoConfig(), (Options::CmdOptList*)m_commandLineParser->GetOptionList(), this);
 	m_options->SetRemoteClientMode(m_commandLineParser->GetRemoteClientMode());
 	m_options->SetServerMode(m_commandLineParser->GetServerMode());
-	m_options->SetPauseDownload(m_commandLineParser->GetPauseDownload());
+	m_workState->SetPauseDownload(m_commandLineParser->GetPauseDownload());
+	m_workState->SetSpeedLimit(g_Options->GetDownloadRate());
 
 	m_log->InitOptions();
 
@@ -412,15 +470,17 @@ void NZBGet::BootConfig()
 		m_commandLineParser->GetClientOperation() == CommandLineParser::opClientNoOperation)
 	{
 		info("Pausing all activities due to errors in configuration");
-		m_options->SetPauseDownload(true);
-		m_options->SetPausePostProcess(true);
-		m_options->SetPauseScan(true);
+		m_workState->SetPauseDownload(true);
+		m_workState->SetPausePostProcess(true);
+		m_workState->SetPauseScan(true);
 	}
 
 	m_serverPool->SetTimeout(m_options->GetArticleTimeout());
 	m_serverPool->SetRetryInterval(m_options->GetArticleInterval());
 
 	m_scriptConfig->InitOptions();
+
+	m_extensionManager->LoadExtensions();
 }
 
 void NZBGet::Cleanup()
@@ -448,46 +508,42 @@ void NZBGet::Cleanup()
 	g_Maintenance = nullptr;
 	g_StatMeter = nullptr;
 	g_CommandScriptLog = nullptr;
+	g_ExtensionManager = nullptr;
+
 #ifdef WIN32
 	g_WinConsole = nullptr;
 #endif
 }
 
-bool NZBGet::ProcessDirect()
+void NZBGet::ProcessDirect()
 {
 #ifdef DEBUG
 	if (m_commandLineParser->GetTestBacktrace())
 	{
-		TestSegFault();
+		TestSegFault(); // never returns
 	}
 #endif
 
 	if (m_commandLineParser->GetWebGet())
 	{
-		ProcessWebGet();
-		return true;
+		ProcessWebGet(); // never returns
 	}
 
 	if (m_commandLineParser->GetSigVerify())
 	{
-		ProcessSigVerify();
-		return true;
+		ProcessSigVerify(); // never returns
 	}
 
 	// client request
 	if (m_commandLineParser->GetClientOperation() != CommandLineParser::opClientNoOperation)
 	{
-		ProcessClientRequest();
-		return true;
+		ProcessClientRequest(); // never returns
 	}
 
 	if (m_commandLineParser->GetPrintOptions())
 	{
-		PrintOptions();
-		return true;
+		PrintOptions(); // never returns
 	}
-
-	return false;
 }
 
 void NZBGet::StartRemoteServer()
@@ -531,7 +587,7 @@ void NZBGet::StopRemoteServer()
 		(m_remoteSecureServer && m_remoteSecureServer->IsRunning())) &&
 		maxWaitMSec > 0)
 	{
-		usleep(100 * 1000);
+		Util::Sleep(100);
 		maxWaitMSec -= 100;
 	}
 
@@ -550,7 +606,7 @@ void NZBGet::StopRemoteServer()
 		(m_remoteSecureServer && m_remoteSecureServer->IsRunning())) &&
 		maxWaitMSec > 0)
 	{
-		usleep(100 * 1000);
+		Util::Sleep(100);
 		maxWaitMSec -= 100;
 	}
 
@@ -608,7 +664,7 @@ void NZBGet::StopFrontend()
 		}
 		while (m_frontend->IsRunning())
 		{
-			usleep(50 * 1000);
+			Util::Sleep(50);
 		}
 		debug("Frontend stopped");
 	}
@@ -625,6 +681,12 @@ void NZBGet::ProcessStandalone()
 		return;
 	}
 	std::unique_ptr<NzbInfo> nzbInfo = nzbFile.DetachNzbInfo();
+
+	if (!nzbFile.GetPassword().empty())
+	{
+		nzbInfo->GetParameters()->SetParameter("*Unpack:Password", nzbFile.GetPassword().c_str());
+	}
+
 	m_scanner->InitPPParameters(category, nzbInfo->GetParameters(), false);
 	m_queueCoordinator->AddNzbFileToQueue(std::move(nzbInfo), nullptr, false);
 }
@@ -688,7 +750,14 @@ void NZBGet::DoMainLoop()
 				m_serviceCoordinator->Stop();
 			}
 		}
-		usleep(100 * 1000);
+		Util::Sleep(100);
+
+		if (m_options->GetServerMode() && !m_stopped)
+		{
+			// wait for stop signal
+			Guard guard(m_waitMutex);
+			m_waitCond.Wait(m_waitMutex, [&]{ return m_stopped; });
+		}
 	}
 
 	debug("Main program loop terminated");
@@ -700,10 +769,7 @@ void NZBGet::Run(bool reload)
 
 	Init();
 
-	if (ProcessDirect())
-	{
-		return;
-	}
+	ProcessDirect();
 
 	StartRemoteServer();
 	StartFrontend();
@@ -722,6 +788,8 @@ void NZBGet::Run(bool reload)
 
 	StopRemoteServer();
 	StopFrontend();
+
+	Final();
 }
 
 void NZBGet::ProcessClientRequest()
@@ -849,7 +917,7 @@ void NZBGet::ProcessWebGet()
 
 void NZBGet::ProcessSigVerify()
 {
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 	bool ok = Maintenance::VerifySignature(m_commandLineParser->GetLastArg(),
 		m_commandLineParser->GetSigFilename(), m_commandLineParser->GetPubKeyFilename());
 	exit(ok ? 93 : 1);
@@ -892,6 +960,11 @@ void NZBGet::Stop(bool reload)
 #endif
 		}
 	}
+
+	// trigger stop/reload signal
+	Guard guard(m_waitMutex);
+	m_stopped = true;
+	m_waitCond.NotifyAll();
 }
 
 void NZBGet::PrintOptions()
@@ -900,6 +973,7 @@ void NZBGet::PrintOptions()
 	{
 		printf("%s = \"%s\"\n", optEntry.GetName(), optEntry.GetValue());
 	}
+	exit(0);
 }
 
 #ifndef WIN32
@@ -932,29 +1006,45 @@ void NZBGet::Daemonize()
 			error("Starting daemon failed: could not create lock-file %s", m_options->GetLockFile());
 			exit(1);
 		}
+
+#ifdef HAVE_LOCKF
 		if (lockf(lfp, F_TLOCK, 0) < 0)
+#else
+		if (flock(lfp, LOCK_EX) < 0)
+#endif
 		{
 			error("Starting daemon failed: could not acquire lock on lock-file %s", m_options->GetLockFile());
 			exit(1);
 		}
 	}
 
+	/* Backward compatibility with QNAP which doesn't have a "root" user,
+	but for historical reasons in nzbget.conf we use "root" as the default value in DaemonUsername
+	which causes problems when running nzbget as a daemon on QNAP. */
+	bool backwardCompRoot = strcmp(m_options->GetDaemonUsername(), "root") == 0;
+
 	/* Drop user if there is one, and we were run as root */
-	if (getuid() == 0 || geteuid() == 0)
+	if (!backwardCompRoot && (getuid() == 0 || geteuid() == 0))
 	{
 		struct passwd *pw = getpwnam(m_options->GetDaemonUsername());
-		if (pw)
+		if (pw == nullptr)
 		{
-			// Change owner of lock file
-			fchown(lfp, pw->pw_uid, pw->pw_gid);
-			// Set aux groups to null.
-			setgroups(0, (const gid_t*)0);
-			// Set primary group.
-			setgid(pw->pw_gid);
-			// Try setting aux groups correctly - not critical if this fails.
-			initgroups(m_options->GetDaemonUsername(), pw->pw_gid);
-			// Finally, set uid.
-			setuid(pw->pw_uid);
+			error("Starting daemon failed: invalid DaemonUsername");
+			exit(1);
+		}
+
+		// Change owner of lock- and logfile
+		chown(m_options->GetLockFile(), pw->pw_uid, pw->pw_gid);
+		chown(m_options->GetLogFile(), pw->pw_uid, pw->pw_gid);
+
+		// Set aux groups to null, configure primary and aux groups, and then assign uid
+		if (setgroups(0, (const gid_t*)0) ||
+				setgid(pw->pw_gid) ||
+				initgroups(m_options->GetDaemonUsername(), pw->pw_gid) ||
+				setuid(pw->pw_uid))
+		{
+			error("Starting daemon failed: could not drop privileges");
+			exit(1);
 		}
 	}
 
@@ -975,10 +1065,11 @@ void NZBGet::Daemonize()
 
 void NZBGet::AddNewsServer(int id, bool active, const char* name, const char* host,
 	int port, int ipVersion, const char* user, const char* pass, bool joinGroup, bool tls,
-	const char* cipher, int maxConnections, int retention, int level, int group, bool optional)
+	const char* cipher, int maxConnections, int retention, int level, int group, bool optional,
+	unsigned int certVerificationfLevel)
 {
 	m_serverPool->AddServer(std::make_unique<NewsServer>(id, active, name, host, port, ipVersion, user, pass, joinGroup,
-		tls, cipher, maxConnections, retention, level, group, optional));
+		tls, cipher, maxConnections, retention, level, group, optional, certVerificationfLevel));
 }
 
 void NZBGet::AddFeed(int id, const char* name, const char* url, int interval, const char* filter,
