@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2023-2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,72 +15,18 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #ifndef NZBGET_H
 #define NZBGET_H
 
+#include "config.h"
+
 /***************** DEFINES FOR WINDOWS *****************/
 #ifdef WIN32
 
-/* Define to 1 to not use curses */
-//#define DISABLE_CURSES
-
-/* Define to 1 to disable smart par-verification and restoration */
-//#define DISABLE_PARCHECK
-
-/* Define to 1 to disable TLS/SSL-support. */
-//#define DISABLE_TLS
-
-#ifndef DISABLE_TLS
-/* Define to 1 to use OpenSSL library for TLS/SSL-support */
-#define HAVE_OPENSSL
-/* Define to 1 to use GnuTLS library for TLS/SSL-support */
-//#define HAVE_LIBGNUTLS
-#endif
-
-/* Define to 1 if OpenSSL supports function "X509_check_host". */
-#define HAVE_X509_CHECK_HOST 1
-
-/* Define to the name of macro which returns the name of function being
-compiled */
-#define FUNCTION_MACRO_NAME __FUNCTION__
-
-/* Define to 1 if ctime_r takes 2 arguments */
-#undef HAVE_CTIME_R_2
-
-/* Define to 1 if ctime_r takes 3 arguments */
-#define HAVE_CTIME_R_3
-
-/* Define to 1 if getopt_long is supported */
-#undef HAVE_GETOPT_LONG
-
-/* Define to 1 if variadic macros are supported */
-#define HAVE_VARIADIC_MACROS
-
-/* Define to 1 if libpar2 supports cancelling (needs a special patch) */
-#define HAVE_PAR2_CANCEL
-
-/* Define to 1 if function GetAddrInfo is supported */
-#define HAVE_GETADDRINFO
-
-/* Determine what socket length (socklen_t) data type is */
-#define SOCKLEN_T socklen_t
-
-/* Define to 1 if you have the <regex.h> header file. */
-#ifndef DISABLE_REGEX
-#define HAVE_REGEX_H 1
-// Static linking to regex library
-#define REGEX_STATIC
-#endif
-
-#ifndef DISABLE_GZIP
-// Static linking to zlib library
-#define ZLIB_WINAPI
-#endif
-
 /* Suppress warnings */
 #define _CRT_SECURE_NO_DEPRECATE
 
@@ -95,6 +42,12 @@ compiled */
 #define _WIN32_WINNT 0x0501
 #endif
 
+#ifdef _WIN64
+#define __amd64__
+#else
+#define __i686__
+#endif
+
 #ifdef _DEBUG
 // detection of memory leaks
 #define _CRTDBG_MAP_ALLOC
@@ -103,16 +56,17 @@ compiled */
 #pragma warning(disable:4800) // 'type' : forcing value to bool 'true' or 'false' (performance warning)
 #pragma warning(disable:4267) // 'var' : conversion from 'size_t' to 'type', possible loss of data
 
-#endif
+#define popen _wpopen
+#define pclose _pclose
 
+#endif
 
 /***************** GLOBAL INCLUDES *****************/
 
 #ifdef WIN32
-
 // WINDOWS INCLUDES
 
-// Using "WIN32_LEAN_AND_MEAN" to disable including on many unneeded headers
+// Using "WIN32_LEAN_AND_MEAN" to disable including of many unneeded headers
 #define WIN32_LEAN_AND_MEAN
 #define NOMINMAX
 
@@ -131,15 +85,7 @@ compiled */
 #include <wincon.h>
 #include <shellapi.h>
 #include <winreg.h>
-
 #include <comutil.h>
-#import <msxml.tlb> named_guids
-using namespace MSXML;
-
-#if _MSC_VER >= 1600
-#include <stdint.h>
-#define HAVE_STDINT_H
-#endif
 
 #ifdef _DEBUG
 #include <crtdbg.h>
@@ -149,16 +95,12 @@ using namespace MSXML;
 
 // POSIX INCLUDES
 
-#include "config.h"
-
 #include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
 #include <signal.h>
 #include <netdb.h>
-#include <unistd.h>
 #include <pthread.h>
-#include <unistd.h>
 #include <getopt.h>
 #include <sys/socket.h>
 #include <sys/time.h>
@@ -166,21 +108,25 @@ using namespace MSXML;
 #include <sys/statvfs.h>
 #include <sys/wait.h>
 #include <sys/un.h>
+#include <sys/file.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
-#include <stdint.h>
-#include <pwd.h>
 #include <dirent.h>
 
-#include <libxml/parser.h>
-#include <libxml/xmlreader.h>
-#include <libxml/xmlerror.h>
-#include <libxml/entities.h>
+#ifndef __linux__
+#include <sys/sysctl.h>
+#endif
+
+#define __BSD__ (__FreeBSD__ || __NetBSD__ || __OpenBSD__ || __DragonFly__)
 
 #ifdef HAVE_SYS_PRCTL_H
 #include <sys/prctl.h>
 #endif
 
+#ifdef HAVE_ENDIAN_H
+#include <endian.h>
+#endif
+
 #ifdef HAVE_BACKTRACE
 #include <execinfo.h>
 #endif
@@ -197,49 +143,64 @@ using namespace MSXML;
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <time.h>
 #include <ctype.h>
+#include <inttypes.h>
 
 #include <string>
+#include <string_view>
 #include <vector>
 #include <deque>
 #include <list>
 #include <set>
 #include <map>
+#include <unordered_map>
 #include <iterator>
 #include <algorithm>
-#include <iostream>
+#include <numeric>
 #include <fstream>
 #include <memory>
 #include <functional>
+#include <thread>
+#include <atomic>
+#include <utility>
+#include <future>
+#include <mutex>
+#include <shared_mutex>
+#include <condition_variable>
+#include <chrono>
+#include <optional>
+#include <variant>
+#include <limits>
+#include <type_traits>
+#include <random>
+#include <exception>
+#include <iomanip>
 
-#ifdef HAVE_LIBGNUTLS
-#ifdef WIN32
-#include <BaseTsd.h>
-typedef SSIZE_T ssize_t;
-typedef int pid_t;
-#endif
-#include <gnutls/gnutls.h>
-#include <gnutls/x509.h>
-#if GNUTLS_VERSION_NUMBER <= 0x020b00
-#define NEED_GCRYPT_LOCKING
-#endif
-#ifdef NEED_GCRYPT_LOCKING
-#include <gcrypt.h>
-#endif /* NEED_GCRYPT_LOCKING */
-#include <nettle/sha.h>
-#include <nettle/pbkdf2.h>
-#include <nettle/aes.h>
-#endif /* HAVE_LIBGNUTLS */
+#include <libxml/parser.h>
+#include <libxml/xmlreader.h>
+#include <libxml/xmlerror.h>
+#include <libxml/entities.h>
+#include <libxml/tree.h>
 
-#ifdef HAVE_OPENSSL
+#include <boost/asio.hpp>
+#ifndef DISABLE_TLS
+#include <boost/asio/ssl.hpp>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/rsa.h>
 #include <openssl/sha.h>
 #include <openssl/pem.h>
 #include <openssl/x509v3.h>
-#endif /* HAVE_OPENSSL */
+#include <openssl/comp.h>
+#endif
+
+// NOTE: do not include <iostream> in "nzbget.h". <iostream> contains objects requiring
+// intialization, causing every unit in nzbget to have initialization routine. This in particular
+// is causing fatal problems in SIMD units which must not have static initialization because
+// they contain code with runtime CPU dispatching.
+//#include <iostream>
 
 #ifdef HAVE_REGEX_H
 #include <regex.h>
@@ -251,14 +212,7 @@ typedef int pid_t;
 
 #ifndef DISABLE_PARCHECK
 #include <assert.h>
-#include <iomanip>
 #include <cassert>
-#ifdef HAVE_MEMORY_H
-# include <memory.h>
-#endif
-#ifdef HAVE_INTTYPES_H
-# include <inttypes.h>
-#endif
 #endif /* NOT DISABLE_PARCHECK */
 
 
@@ -268,7 +222,6 @@ typedef int pid_t;
 
 // WINDOWS
 
-#define snprintf _snprintf
 #ifndef strdup
 #define strdup _strdup
 #endif
@@ -279,13 +232,12 @@ typedef int pid_t;
 #define strerror_r(errnum, buffer, size) strerror_s(buffer, size, errnum)
 #define mkdir(dir, flags) _mkdir(dir)
 #define rmdir _rmdir
-#define strcasecmp(a, b) _stricmp(a, b)
-#define strncasecmp(a, b, c) _strnicmp(a, b, c)
+#define strcasecmp(a, b) stricmp(a, b)
+#define strncasecmp(a, b, c) strnicmp(a, b, c)
 #define __S_ISTYPE(mode, mask) (((mode) & _S_IFMT) == (mask))
 #define S_ISDIR(mode) __S_ISTYPE((mode), _S_IFDIR)
 #define S_ISREG(mode) __S_ISTYPE((mode), _S_IFREG)
 #define S_DIRMODE nullptr
-#define usleep(usec) Sleep((usec) / 1000)
 #define socklen_t int
 #define SHUT_WR 0x01
 #define SHUT_RDWR 0x02
@@ -307,18 +259,15 @@ typedef int pid_t;
 #define FOPEN_WB "wbN"
 #define FOPEN_AB "abN"
 
+#define __SSE2__
+#define __SSSE3__
+#define __PCLMUL__
+
 #ifdef DEBUG
 // redefine "exit" to avoid printing memory leaks report when terminated because of wrong command line switches
 #define exit(code) ExitProcess(code)
 #endif
 
-#ifdef HAVE_OPENSSL
-FILE _iob[] = {*stdin, *stdout, *stderr};
-extern "C" FILE * __cdecl __iob_func(void) { return _iob; }
-// For static linking of OpenSSL libraries:
-#pragma comment (lib, "legacy_stdio_definitions.lib")
-#endif /* HAVE_OPENSSL */
-
 #else
 
 // POSIX
@@ -336,6 +285,7 @@ extern "C" FILE * __cdecl __iob_func(void) { return _iob; }
 #define FOPEN_WB "wb"
 #define FOPEN_AB "ab"
 #define CHILD_WATCHDOG 1
+#define fseek fseeko
 
 #endif /* POSIX */
 
@@ -348,7 +298,7 @@ extern "C" FILE * __cdecl __iob_func(void) { return _iob; }
 typedef uint8_t uint8;
 typedef int16_t int16;
 typedef uint16_t uint16;
-typedef uint32_t int32;
+typedef int32_t int32;
 typedef uint32_t uint32;
 typedef int64_t int64;
 typedef uint64_t uint64;
@@ -362,8 +312,25 @@ typedef signed long long int64;
 typedef unsigned long long uint64;
 #endif
 
+#ifndef PRId64
+#define PRId64 "lld"
+#endif
+#ifndef PRIi64
+#define PRIi64 "lli"
+#endif
+#ifndef PRIu64
+#define PRIu64 "llu"
+#endif
+
 typedef unsigned char uchar;
 
+// Assume little endian if byte order is not defined
+#ifndef __BYTE_ORDER
+#define __LITTLE_ENDIAN 1234
+#define __BIG_ENDIAN    4321
+#define __BYTE_ORDER __LITTLE_ENDIAN
+#endif
+
 #ifdef __GNUC__
 #define PRINTF_SYNTAX(strindex) __attribute__ ((format (printf, strindex, strindex+1)))
 #define SCANF_SYNTAX(strindex) __attribute__ ((format (scanf, strindex, strindex+1)))
