@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
  *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -39,6 +40,7 @@ public:
 	void InitConnections();
 	int GetMaxNormLevel() { return m_maxNormLevel; }
 	Servers* GetServers() { return &m_servers; } // Only for read access (no lockings)
+	NewsServer* GetServerById(int id);
 	NntpConnection* GetConnection(int level, NewsServer* wantServer, RawServerList* ignoreServers);
 	void FreeConnection(NntpConnection* connection, bool used);
 	void CloseUnusedConnections();
