@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024-2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,7 +56,7 @@ void DirectUnpack::Run()
 		m_destDir = nzbInfo->GetDestDir();
 		m_finalDir = nzbInfo->BuildFinalDirName();
 
-		NzbParameter* parameter = nzbInfo->GetParameters()->Find("*Unpack:Password", false);
+		NzbParameter* parameter = nzbInfo->GetParameters()->Find("*Unpack:Password");
 		if (parameter)
 		{
 			m_password = parameter->GetValue();
@@ -105,7 +106,7 @@ void DirectUnpack::Run()
 			{
 				break;
 			}
-			usleep(100 * 1000);
+			Util::Sleep(100);
 		}
 	}
 
@@ -180,7 +181,7 @@ void DirectUnpack::ExecuteUnrar(const char* archiveName)
 		params.emplace_back("-p-");
 	}
 
-	if (!params.Exists("-o+") && !params.Exists("-o-"))
+	if (!params.Exists("-o+") && !params.Exists("-o-") && !params.Exists("-or"))
 	{
 		params.emplace_back("-o+");
 	}
@@ -394,7 +395,7 @@ void DirectUnpack::AddMessage(Message::EKind kind, const char* text)
 
 void DirectUnpack::Stop(DownloadQueue* downloadQueue, NzbInfo* nzbInfo)
 {
-	debug("Stopping direct unpack for %s", *m_infoName);
+	debug("Stopping %s", *m_infoName);
 	if (m_processed)
 	{
 		if (nzbInfo)
@@ -458,7 +459,7 @@ void DirectUnpack::FileDownloaded(DownloadQueue* downloadQueue, FileInfo* fileIn
 {
 	debug("FileDownloaded for %s/%s", fileInfo->GetNzbInfo()->GetName(), fileInfo->GetFilename());
 
-	if (fileInfo->GetNzbInfo()->GetFailedArticles() > 0)
+	if (!CanProceed(fileInfo))
 	{
 		Stop(downloadQueue, fileInfo->GetNzbInfo());
 		return;
@@ -477,6 +478,14 @@ void DirectUnpack::FileDownloaded(DownloadQueue* downloadQueue, FileInfo* fileIn
 	}
 }
 
+bool DirectUnpack::CanProceed(FileInfo* fileInfo) const
+{
+	if (!IsArchiveFilename(fileInfo->GetFilename())) 
+		return true;
+
+	return fileInfo->GetFailedArticles() == 0;
+}
+
 void DirectUnpack::NzbDownloaded(DownloadQueue* downloadQueue, NzbInfo* nzbInfo)
 {
 	debug("NzbDownloaded for %s", nzbInfo->GetName());
