@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -173,7 +174,7 @@ bool RarVolume::Skip(DiskFile& file, RarBlock* block, int64 size)
 	uint8 buf[256];
 	while (size > 0)
 	{
-		int64 len = size <= sizeof(buf) ? size : sizeof(buf);
+		int64 len = size <= (int64)sizeof(buf) ? size : (int64)sizeof(buf);
 		if (!Read(file, block, buf, len)) return false;
 		size -= len;
 	}
@@ -249,19 +250,19 @@ bool RarVolume::ReadRar3Volume(DiskFile& file)
 
 RarVolume::RarBlock RarVolume::ReadRar3Block(DiskFile& file)
 {
-	RarBlock block {0};
+	RarBlock block{};
 	uint8 salt[8];
 
 	if (m_encrypted &&
 		!(file.Read(salt, sizeof(salt)) == sizeof(salt) &&
 		  DecryptRar3Prepare(salt) && DecryptInit(128)))
 	{
-		return {0};
+		return {};
 	}
 
 	uint8 buf[7];
 
-	if (!Read(file, nullptr, &buf, sizeof(buf))) return {0};
+	if (!Read(file, nullptr, &buf, sizeof(buf))) return {};
 	block.crc = ((uint16)buf[1] << 8) + buf[0];
 	block.type = buf[2];
 	block.flags = ((uint16)buf[4] << 8) + buf[3];
@@ -279,7 +280,7 @@ RarVolume::RarBlock RarVolume::ReadRar3Block(DiskFile& file)
 	uint8 addbuf[4];
 	if ((block.flags & RAR3_BLOCK_ADDSIZE) && !Read(file, nullptr, &addbuf, sizeof(addbuf)))
 	{
-		return {0};
+		return {};
 	}
 	block.addsize = ((uint32)addbuf[3] << 24) + ((uint32)addbuf[2] << 16) + ((uint32)addbuf[1] << 8) + addbuf[0];
 
@@ -289,8 +290,10 @@ RarVolume::RarBlock RarVolume::ReadRar3Block(DiskFile& file)
 		block.trailsize = blocksize - sizeof(buf) - 4;
 	}
 
+#ifdef DEBUG
 	static int num = 0;
-	debug("%i) %llu, %i, %i, %i, %u, %llu", ++num, (long long)block.crc, (int)block.type, (int)block.flags, (int)size, (int)block.addsize, (long long)block.trailsize);
+	debug("%i) %u, %i, %i, %i, %" PRIu64 ", %" PRIu64, ++num, block.crc, block.type, block.flags, size, block.addsize, block.trailsize);
+#endif
 
 	return block;
 }
@@ -421,37 +424,39 @@ bool RarVolume::ReadRar5Volume(DiskFile& file)
 
 RarVolume::RarBlock RarVolume::ReadRar5Block(DiskFile& file)
 {
-	RarBlock block {0};
+	RarBlock block{};
 	uint64 buf = 0;
 
 	if (m_encrypted &&
 		!(file.Read(m_decryptIV, sizeof(m_decryptIV)) == sizeof(m_decryptIV) &&
 		  DecryptInit(256)))
 	{
-		return {0};
+		return {};
 	}
 
-	if (!Read32(file, nullptr, &block.crc)) return {0};
+	if (!Read32(file, nullptr, &block.crc)) return {};
 
-	if (!ReadV(file, nullptr, &buf)) return {0};
+	if (!ReadV(file, nullptr, &buf)) return {};
 	uint32 size = (uint32)buf;
 	block.trailsize = size;
 
-	if (!ReadV(file, &block, &buf)) return {0};
+	if (!ReadV(file, &block, &buf)) return {};
 	block.type = (uint8)buf;
 
-	if (!ReadV(file, &block, &buf)) return {0};
+	if (!ReadV(file, &block, &buf)) return {};
 	block.flags = (uint16)buf;
 
 	block.addsize = 0;
-	if ((block.flags & RAR5_BLOCK_EXTRADATA) && !ReadV(file, &block, &block.addsize)) return {0};
+	if ((block.flags & RAR5_BLOCK_EXTRADATA) && !ReadV(file, &block, &block.addsize)) return {};
 
 	uint64 datasize = 0;
-	if ((block.flags & RAR5_BLOCK_DATAAREA) && !ReadV(file, &block, &datasize)) return {0};
+	if ((block.flags & RAR5_BLOCK_DATAAREA) && !ReadV(file, &block, &datasize)) return {};
 	block.trailsize += datasize;
 
+#ifdef DEBUG
 	static int num = 0;
-	debug("%i) %llu, %i, %i, %i, %u, %llu", ++num, (long long)block.crc, (int)block.type, (int)block.flags, (int)size, (int)block.addsize, (long long)block.trailsize);
+	debug("%i) %u, %i, %i, %i, %" PRIu64 ", %" PRIu64, ++num, block.crc, block.type, block.flags, size, block.addsize, block.trailsize);
+#endif
 
 	return block;
 }
@@ -527,23 +532,25 @@ bool RarVolume::ReadRar5File(DiskFile& file, RarBlock& block, RarFile& innerFile
 		}
 	}
 
-	debug("%llu, %i, %s", (long long)block.trailsize, (int)namelen, (const char*)name);
+	debug("%" PRIu64 ", %" PRIu64 ", %s", block.trailsize, namelen, (const char*)name);
 
 	return true;
 }
 
 void RarVolume::LogDebugInfo()
 {
+#ifdef DEBUG
 	debug("Volume: version:%i, multi:%i, vol-no:%i, new-naming:%i, has-next:%i, encrypted:%i, file-count:%i, [%s]",
 		(int)m_version, (int)m_multiVolume, m_volumeNo, (int)m_newNaming, (int)m_hasNextVolume,
 		(int)m_encrypted, (int)m_files.size(), FileSystem::BaseFileName(m_filename));
 
 	for (RarFile& file : m_files)
 	{
-		debug("  time:%i, size:%lli, attr:%i, split-before:%i, split-after:%i, [%s]",
-			(int)file.m_time, (long long)file.m_size, (int)file.m_attr,
-			(int)file.m_splitBefore, (int)file.m_splitAfter, *file.m_filename);
+		debug("  time:%i, size:%" PRIi64 ", attr:%i, split-before:%i, split-after:%i, [%s]",
+			file.m_time, file.m_size, file.m_attr,
+			file.m_splitBefore, file.m_splitAfter, *file.m_filename);
 	}
+#endif
 }
 
 bool RarVolume::DecryptRar3Prepare(const uint8 salt[8])
@@ -563,7 +570,7 @@ bool RarVolume::DecryptRar3Prepare(const uint8 salt[8])
 
 	debug("seed: %s", *Util::FormatBuffer((const char*)seed, seed.Size()));
 
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 	EVP_MD_CTX* context = EVP_MD_CTX_create();
 
 	if (!EVP_DigestInit(context, EVP_sha1()))
@@ -571,9 +578,6 @@ bool RarVolume::DecryptRar3Prepare(const uint8 salt[8])
 		EVP_MD_CTX_destroy(context);
 		return false;
 	}
-#elif defined(HAVE_NETTLE)
-	sha1_ctx context;
-	sha1_init(&context);
 #else
 	return false;
 #endif
@@ -583,10 +587,8 @@ bool RarVolume::DecryptRar3Prepare(const uint8 salt[8])
 
 	for (int i = 0; i < rounds; i++)
 	{
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 		EVP_DigestUpdate(context, *seed, seed.Size());
-#elif defined(HAVE_NETTLE)
-		sha1_update(&context, seed.Size(), (const uint8_t*)*seed);
 #endif
 
 		uint8 buf[3];
@@ -594,32 +596,25 @@ bool RarVolume::DecryptRar3Prepare(const uint8 salt[8])
 		buf[1] = (uint8)(i >> 8);
 		buf[2] = (uint8)(i >> 16);
 
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 		EVP_DigestUpdate(context, buf, sizeof(buf));
-#elif defined(HAVE_NETTLE)
-		sha1_update(&context, sizeof(buf), buf);
 #endif
 
 		if (i % (rounds / 16) == 0)
 		{
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 			EVP_MD_CTX* ivContext = EVP_MD_CTX_create();
 			EVP_MD_CTX_copy(ivContext, context);
 			EVP_DigestFinal(ivContext, digest, nullptr);
 			EVP_MD_CTX_destroy(ivContext);
-#elif defined(HAVE_NETTLE)
-			sha1_ctx ivContext = context;
-			sha1_digest(&ivContext, sizeof(digest), digest);
 #endif
 			m_decryptIV[i / (rounds / 16)] = digest[sizeof(digest) - 1];
 		}
 	}
 
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 	EVP_DigestFinal(context, digest, nullptr);
 	EVP_MD_CTX_destroy(context);
-#elif defined(HAVE_NETTLE)
-	sha1_digest(&context, sizeof(digest), digest);
 #endif
 
 	debug("digest: %s", *Util::FormatBuffer((const char*)digest, sizeof(digest)));
@@ -644,14 +639,10 @@ bool RarVolume::DecryptRar5Prepare(uint8 kdfCount, const uint8 salt[16])
 
 	int iterations = 1 << kdfCount;
 
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 	if (!PKCS5_PBKDF2_HMAC(m_password, m_password.Length(), salt, 16,
 		iterations, EVP_sha256(), sizeof(m_decryptKey), m_decryptKey)) return false;
 	return true;
-#elif defined(HAVE_NETTLE)
-	pbkdf2_hmac_sha256(m_password.Length(), (const uint8_t*)*m_password,
-		iterations, 16, salt, sizeof(m_decryptKey), m_decryptKey);
-	return true;
 #else
 	return false;
 #endif
@@ -659,17 +650,13 @@ bool RarVolume::DecryptRar5Prepare(uint8 kdfCount, const uint8 salt[16])
 
 bool RarVolume::DecryptInit(int keyLength)
 {
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 	if (!(m_context = EVP_CIPHER_CTX_new())) return false;
 	if (!EVP_DecryptInit((EVP_CIPHER_CTX*)m_context,
 		keyLength == 128 ? EVP_aes_128_cbc() : EVP_aes_256_cbc(),
 		m_decryptKey, m_decryptIV))
 		return false;
 	return true;
-#elif defined(HAVE_NETTLE)
-	m_context = new aes_ctx;
-	aes_set_decrypt_key((aes_ctx*)m_context, keyLength == 128 ? 16 : 32, m_decryptKey);
-	return true;
 #else
 	return false;
 #endif
@@ -677,22 +664,13 @@ bool RarVolume::DecryptInit(int keyLength)
 
 bool RarVolume::DecryptBuf(const uint8 in[16], uint8 out[16])
 {
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 	uint8 outbuf[32];
 	int outlen = 0;
 	if (!EVP_DecryptUpdate((EVP_CIPHER_CTX*)m_context, outbuf, &outlen, in, 16)) return false;
 	memcpy(out, outbuf + outlen, 16);
 	debug("decrypted: %s", *Util::FormatBuffer((const char*)out, 16));
 	return true;
-#elif defined(HAVE_NETTLE)
-	aes_decrypt((aes_ctx*)m_context, 16, out, in);
-	for (int i = 0; i < 16; i++)
-	{
-		out[i] ^= m_decryptIV[i];
-	}
-	memcpy(m_decryptIV, in, 16);
-	debug("decrypted: %s", *Util::FormatBuffer((const char*)out, 16));
-	return true;
 #else
 	return false;
 #endif
@@ -702,10 +680,8 @@ void RarVolume::DecryptFree()
 {
 	if (m_context)
 	{
-#ifdef HAVE_OPENSSL
+#ifndef DISABLE_TLS
 		EVP_CIPHER_CTX_free((EVP_CIPHER_CTX*)m_context);
-#elif defined(HAVE_NETTLE)
-		delete (aes_ctx*)m_context;
 #endif
 		m_context = nullptr;
 	}
