@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
- *
- *  Copyright (C) 2013-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
+ *  Copyright (C) 2013-2018 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2023-2025 Denis <denis@nzbget.com>
+ * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -60,10 +61,10 @@ void UnpackController::Run()
 		m_finalDir = m_postInfo->GetNzbInfo()->GetFinalDir();
 		m_name = m_postInfo->GetNzbInfo()->GetName();
 
-		NzbParameter* parameter = m_postInfo->GetNzbInfo()->GetParameters()->Find("*Unpack:", false);
+		NzbParameter* parameter = m_postInfo->GetNzbInfo()->GetParameters()->Find("*Unpack:");
 		unpack = !(parameter && !strcasecmp(parameter->GetValue(), "no"));
 
-		parameter = m_postInfo->GetNzbInfo()->GetParameters()->Find("*Unpack:Password", false);
+		parameter = m_postInfo->GetNzbInfo()->GetParameters()->Find("*Unpack:Password");
 		if (parameter)
 		{
 			m_password = parameter->GetValue();
@@ -284,7 +285,7 @@ void UnpackController::ExecuteUnrar(const char* password)
 		params.emplace_back("-p-");
 	}
 
-	if (!params.Exists("-o+") && !params.Exists("-o-"))
+	if (!params.Exists("-o+") && !params.Exists("-o-") && !params.Exists("-or"))
 	{
 		params.emplace_back("-o+");
 	}
@@ -649,11 +650,10 @@ void UnpackController::CheckArchiveFiles()
 
 			if (regExRar.Match(filename))
 			{
+				const auto begin = m_postInfo->GetExtractedArchives()->cbegin();
+				const auto end = m_postInfo->GetExtractedArchives()->cend();
 				m_hasRarFiles = true;
-				m_hasNotUnpackedRarFiles |= std::find(
-					m_postInfo->GetExtractedArchives()->begin(),
-					m_postInfo->GetExtractedArchives()->end(),
-					filename) == m_postInfo->GetExtractedArchives()->end();
+				m_hasNotUnpackedRarFiles |= std::find(begin, end, filename) == end;
 			}
 			else if (regExSevenZip.Match(filename))
 			{
@@ -682,7 +682,7 @@ bool UnpackController::FileHasRarSignature(const char* filename)
 	char rar4Signature[] = { 0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x00 };
 	char rar5Signature[] = { 0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x01, 0x00 };
 
-	char fileSignature[8];
+	char fileSignature[8]{};
 
 	int cnt = 0;
 	DiskFile infile;
@@ -714,11 +714,11 @@ bool UnpackController::Cleanup()
 	{
 		// moving files back
 		DirBrowser dir(m_unpackDir);
+		const char* destDir = !m_finalDir.Empty() ? *m_finalDir : *m_destDir;
 		while (const char* filename = dir.Next())
 		{
 			BString<1024> srcFile("%s%c%s", *m_unpackDir, PATH_SEPARATOR, filename);
-			BString<1024> dstFile("%s%c%s", !m_finalDir.Empty() ? *m_finalDir : *m_destDir,
-				PATH_SEPARATOR, *FileSystem::MakeValidFilename(filename));
+			BString<1024> dstFile("%s%c%s", destDir, PATH_SEPARATOR, *FileSystem::MakeValidFilename(filename));
 
 			// silently overwrite existing files
 			FileSystem::DeleteFile(dstFile);
@@ -732,8 +732,19 @@ bool UnpackController::Cleanup()
 				ok = false;
 			}
 
+#ifndef WIN32
+			// Fixing file or directory permissions overridden by the unpacker
+			FileSystem::RestoreFileOrDirPermissions(dstFile.Str());
+#endif
+
 			extractedFiles.push_back(filename);
 		}
+
+#ifndef WIN32
+		// Fixing directory permissions overridden by the unpacker
+		FileSystem::RestoreDirPermissions(destDir);
+#endif
+
 	}
 
 	CString errmsg;
@@ -857,9 +868,12 @@ void UnpackController::AddMessage(Message::EKind kind, const char* text)
 		msgText.Format("Unrar: Extracting %s", text + 19 + strlen(m_unpackExtendedDir) + 1);
 	}
 
-	m_postInfo->GetNzbInfo()->AddMessage(kind, msgText);
+	if (m_postInfo)
+	{
+		m_postInfo->GetNzbInfo()->AddMessage(kind, msgText);
+	}
 
-	if (m_unpacker == upUnrar && !strncmp(msgText, "Unrar: UNRAR ", 6) &&
+	if (m_postInfo && m_unpacker == upUnrar && !strncmp(msgText, "Unrar: UNRAR ", 13) &&
 		strstr(msgText, " Copyright ") && strstr(msgText, " Alexander Roshal"))
 	{
 		// reset start time for a case if user uses unpack-script to do some things
@@ -874,8 +888,10 @@ void UnpackController::AddMessage(Message::EKind kind, const char* text)
 
 	if (m_unpacker == upUnrar && !strncmp(text, "Unrar: Extracting from ", 23))
 	{
+#ifdef DEBUG
 		const char *filename = text + 23;
 		debug("Filename: %s", filename);
+#endif
 		SetProgressLabel(text + 7);
 	}
 
@@ -886,13 +902,14 @@ void UnpackController::AddMessage(Message::EKind kind, const char* text)
 		m_unpackDecryptError = true;
 	}
 
-	if (m_unpacker == upUnrar && !strncmp(text, "Unrar: The specified password is incorrect.", 43))
+	if (m_unpacker == upUnrar && (!strncmp(text, "Unrar: The specified password is incorrect.", 43) ||
+		!strncmp(text, "Unrar: Incorrect password for", 29)))
 	{
 		m_unpackPasswordError = true;
 	}
 
 	if (m_unpacker == upSevenZip &&
-		(len > 18 && !strncmp(text + len - 45, "Data Error in encrypted file. Wrong password?", 45)))
+		(len > 45 && !strncmp(text + len - 45, "Data Error in encrypted file. Wrong password?", 45)))
 	{
 		m_unpackDecryptError = true;
 	}
@@ -904,8 +921,14 @@ void UnpackController::AddMessage(Message::EKind kind, const char* text)
 		(len > 18 && !strncmp(text + len - 18, " - checksum failed", 18)) ||
 		!strncmp(text, "Unrar: WARNING: You need to start extraction from a previous volume", 67)))
 	{
-		m_postInfo->GetNzbInfo()->AddMessage(Message::mkWarning,
-			BString<1024>("Cancelling %s due to errors", *m_infoName));
+		if (m_postInfo)
+		{
+			m_postInfo->GetNzbInfo()->AddMessage(
+				Message::mkWarning,
+				BString<1024>("Cancelling %s due to errors", *m_infoName)
+			);
+		}
+
 		m_autoTerminated = true;
 		Stop();
 	}
@@ -929,3 +952,53 @@ void UnpackController::SetProgressLabel(const char* progressLabel)
 	GuardedDownloadQueue guard = DownloadQueue::Guard();
 	m_postInfo->SetProgressLabel(progressLabel);
 }
+
+bool UnpackController::HasCompletedArchiveFiles(NzbInfo* nzbInfo)
+{
+	RegEx regExRar(".*\\.rar$");
+	RegEx regExSevenZip(".*\\.7z$");
+	RegEx regExSevenZipMulti(".*\\.7z\\.[0-9]+$");
+
+	for (CompletedFile& completedFile: nzbInfo->GetCompletedFiles())
+	{
+		const char* filename = completedFile.GetFilename();
+		if (regExRar.Match(filename) ||
+			regExSevenZip.Match(filename) ||
+			regExSevenZipMulti.Match(filename))
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+const char* UnpackController::DecodeSevenZipExitCode(int ec)
+{
+	// 7-Zip exit codes according to https://documentation.help/7-Zip/exit_codes.htm
+	switch (ec)
+	{
+	case SevenZipExitCodes::NoError:
+		return "No error";
+
+	case SevenZipExitCodes::Warning:
+		return "Warning (Non fatal error(s)). \
+		For example, one or more files were locked by some other application, \
+		so they were not compressed.";
+
+	case SevenZipExitCodes::FatalError:
+		return "Fatal error";
+
+	case SevenZipExitCodes::CmdLineError:
+		return "Command line error";
+
+	case SevenZipExitCodes::NotEnoughMemoryError:
+		return "Not enough memory for operation";
+
+	case SevenZipExitCodes::CanceledByUser:
+		return "User stopped the process";
+
+	default: 
+		return "Unknown 7-Zip error";
+	};
+}
