@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2013-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2013-2018 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "Log.h"
 #include "Thread.h"
 #include "DownloadInfo.h"
-#include "Script.h"
+#include "ScriptController.h"
 
 class UnpackController : public Thread, public ScriptController
 {
@@ -32,6 +32,8 @@ public:
 	virtual void Run();
 	virtual void Stop();
 	static void StartJob(PostInfo* postInfo);
+	static bool HasCompletedArchiveFiles(NzbInfo* nzbInfo);
+	static const char* DecodeSevenZipExitCode(int ec);
 
 protected:
 	virtual bool ReadLine(char* buf, int bufSize, FILE* stream);
@@ -44,6 +46,16 @@ private:
 		upSevenZip
 	};
 
+	enum SevenZipExitCodes
+	{
+		NoError = 0,
+		Warning = 1,
+		FatalError = 2,
+		CmdLineError = 7,
+		NotEnoughMemoryError = 8,
+		CanceledByUser = 255,
+	};
+
 	typedef std::vector<CString> FileListBase;
 	class FileList : public FileListBase
 	{
@@ -58,7 +70,7 @@ private:
 		bool Exists(const char* param);
 	};
 
-	PostInfo* m_postInfo;
+	PostInfo* m_postInfo = nullptr;
 	CString m_name;
 	CString m_infoName;
 	CString m_infoNameUp;
