@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024-2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -27,9 +28,9 @@
 #include "FileSystem.h"
 
 static const char* FORMATVERSION_SIGNATURE = "nzbget diskstate file version ";
-const int DISKSTATE_QUEUE_VERSION = 60;
-const int DISKSTATE_FILE_VERSION = 5;
-const int DISKSTATE_STATS_VERSION = 3;
+const int DISKSTATE_QUEUE_VERSION = 62;
+const int DISKSTATE_FILE_VERSION = 7;
+const int DISKSTATE_STATS_VERSION = 4;
 const int DISKSTATE_FEEDS_VERSION = 3;
 
 class StateDiskFile : public DiskFile
@@ -45,12 +46,10 @@ int64 StateDiskFile::PrintLine(const char* format, ...)
 {
 	va_list ap;
 	va_start(ap, format);
-	CString str;
-	str.FormatV(format, ap);
+	BString<1024> str;
+	int len = str.FormatV(format, ap);
 	va_end(ap);
 
-	int len = str.Length();
-
 	// replacing terminating <NULL> with <LF>
 	str[len++] = '\n';
 
@@ -117,8 +116,8 @@ public:
 private:
 	BString<1024> m_destFilename;
 	BString<1024> m_tempFilename;
-	bool m_transactional;
 	int m_formatVersion;
+	bool m_transactional;
 	int m_fileVersion;
 	StateDiskFile m_file;
 
@@ -313,6 +312,10 @@ bool DiskState::SaveDownloadQueue(DownloadQueue* downloadQueue, bool saveHistory
 		}
 	}
 
+	// progress-file isn't needed after saving of full queue data
+	StateFile progressStateFile("progress", DISKSTATE_QUEUE_VERSION, true);
+	progressStateFile.Discard();
+
 	return ok;
 }
 
@@ -335,7 +338,12 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* downloadQueue, Servers* servers
 
 			formatVersion = stateFile.GetFileVersion();
 
-			if (formatVersion < 47)
+			if (formatVersion <= 0)
+			{
+				error("Failed to read queue: diskstate file is corrupted");
+				goto error;
+			}
+			else if (formatVersion < 47)
 			{
 				error("Failed to read queue and history data. Only queue and history from NZBGet v13 or newer can be converted by this NZBGet version. "
 					"Old queue and history data still can be converted using NZBGet v16 as an intermediate version.");
@@ -351,6 +359,26 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* downloadQueue, Servers* servers
 		}
 	}
 
+	{
+		StateFile stateFile("progress", DISKSTATE_QUEUE_VERSION, true);
+		if (stateFile.FileExists())
+		{
+			StateDiskFile* infile = stateFile.BeginRead();
+			if (!infile)
+			{
+				return false;
+			}
+
+			if (stateFile.GetFileVersion() <= 0)
+			{
+				error("Failed to read queue: diskstate file is corrupted");
+				goto error;
+			}
+
+			if (!LoadProgress(downloadQueue->GetQueue(), servers, *infile, stateFile.GetFileVersion())) goto error;
+		}
+	}
+
 	if (formatVersion == 0 || formatVersion >= 57)
 	{
 		StateFile stateFile("history", DISKSTATE_QUEUE_VERSION, true);
@@ -361,6 +389,13 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* downloadQueue, Servers* servers
 			{
 				return false;
 			}
+
+			if (stateFile.GetFileVersion() <= 0)
+			{
+				error("Failed to read queue: diskstate file is corrupted");
+				goto error;
+			}
+
 			if (!LoadHistory(downloadQueue->GetHistory(), servers, *infile, stateFile.GetFileVersion())) goto error;
 		}
 	}
@@ -388,6 +423,42 @@ error:
 	return ok;
 }
 
+bool DiskState::SaveDownloadProgress(DownloadQueue* downloadQueue)
+{
+	int count = 0;
+	for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
+	{
+		count += nzbInfo->GetChanged() ? 1 : 0;
+	}
+
+	debug("Saving queue progress to disk");
+
+	bool ok = true;
+
+	{
+		StateFile stateFile("progress", DISKSTATE_QUEUE_VERSION, true);
+		if (count > 0)
+		{
+			StateDiskFile* outfile = stateFile.BeginWrite();
+			if (!outfile)
+			{
+				return false;
+			}
+
+			SaveProgress(downloadQueue->GetQueue(), *outfile, count);
+
+			// now rename to dest file name
+			ok = stateFile.FinishWrite();
+		}
+		else
+		{
+			stateFile.Discard();
+		}
+	}
+
+	return ok;
+}
+
 void DiskState::SaveQueue(NzbList* queue, StateDiskFile& outfile)
 {
 	debug("Saving nzb list to disk");
@@ -420,6 +491,50 @@ error:
 	return false;
 }
 
+void DiskState::SaveProgress(NzbList* queue, StateDiskFile& outfile, int changedCount)
+{
+	debug("Saving nzb progress to disk");
+
+	outfile.PrintLine("%i", changedCount);
+	for (NzbInfo* nzbInfo : queue)
+	{
+		if (nzbInfo->GetChanged())
+		{
+			outfile.PrintLine("%i", nzbInfo->GetId());
+			SaveNzbInfo(nzbInfo, outfile);
+		}
+	}
+}
+
+bool DiskState::LoadProgress(NzbList* queue, Servers* servers, StateDiskFile& infile, int formatVersion)
+{
+	debug("Loading nzb progress from disk");
+
+	// load nzb-infos
+	int size;
+	if (infile.ScanLine("%i", &size) != 1) goto error;
+	for (int i = 0; i < size; i++)
+	{
+		int id;
+		if (infile.ScanLine("%i", &id) != 1) goto error;
+
+		NzbInfo* nzbInfo = queue->Find(id);
+		if (!nzbInfo)
+		{
+			error("NZB with id %i could not be found", id);
+			goto error;
+		}
+
+		if (!LoadNzbInfo(nzbInfo, servers, infile, formatVersion)) goto error;
+	}
+
+	return true;
+
+error:
+	error("Error reading nzb progress from disk");
+	return false;
+}
+
 void DiskState::SaveNzbInfo(NzbInfo* nzbInfo, StateDiskFile& outfile)
 {
 	outfile.PrintLine("%i", nzbInfo->GetId());
@@ -464,7 +579,7 @@ void DiskState::SaveNzbInfo(NzbInfo* nzbInfo, StateDiskFile& outfile)
 	outfile.PrintLine("%i,%i,%i", nzbInfo->GetTotalArticles(), nzbInfo->GetSuccessArticles(), nzbInfo->GetFailedArticles());
 
 	outfile.PrintLine("%s", nzbInfo->GetDupeKey());
-	outfile.PrintLine("%i,%i", (int)nzbInfo->GetDupeMode(), nzbInfo->GetDupeScore());
+	outfile.PrintLine("%i,%i,%i", (int)nzbInfo->GetDupeMode(), nzbInfo->GetDupeScore(), (int)nzbInfo->GetDupeHint());
 
 	Util::SplitInt64(nzbInfo->GetDownloadedSize(), &High1, &Low1);
 	outfile.PrintLine("%u,%u,%i,%i,%i,%i,%i", High1, Low1, nzbInfo->GetDownloadSec(), nzbInfo->GetPostTotalSec(),
@@ -473,14 +588,21 @@ void DiskState::SaveNzbInfo(NzbInfo* nzbInfo, StateDiskFile& outfile)
 	outfile.PrintLine("%i", (int)nzbInfo->GetCompletedFiles()->size());
 	for (CompletedFile& completedFile : nzbInfo->GetCompletedFiles())
 	{
-		outfile.PrintLine("%i,%i,%u,%i,%s,%s,%s", completedFile.GetId(), (int)completedFile.GetStatus(),
+		outfile.PrintLine("%i,%i,%u,%i,%s,%s", completedFile.GetId(), (int)completedFile.GetStatus(),
 			completedFile.GetCrc(), (int)completedFile.GetParFile(),
 			completedFile.GetHash16k() ? completedFile.GetHash16k() : "",
-			completedFile.GetParSetId() ? completedFile.GetParSetId() : "",
-			completedFile.GetFilename());
+			completedFile.GetParSetId() ? completedFile.GetParSetId() : "");
+		outfile.PrintLine("%s", completedFile.GetFilename());
+		outfile.PrintLine("%s", completedFile.GetOrigname() ? completedFile.GetOrigname() : "");
 	}
 
-	outfile.PrintLine("%i", (int)nzbInfo->GetParameters()->size());
+	outfile.PrintLine(
+		"%i,%i,%i,%i", 
+		static_cast<int>(nzbInfo->GetParameters()->size()),
+		static_cast<int>(nzbInfo->GetSkipScriptProcessing()),
+		static_cast<int>(nzbInfo->GetSkipDiskWrite()),
+		nzbInfo->GetDesiredServerId()
+	);
 	for (NzbParameter& parameter : nzbInfo->GetParameters())
 	{
 		outfile.PrintLine("%s=%s", parameter.GetName(), parameter.GetValue());
@@ -517,6 +639,11 @@ void DiskState::SaveNzbInfo(NzbInfo* nzbInfo, StateDiskFile& outfile)
 bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& infile, int formatVersion)
 {
 	char buf[10240];
+	int parameterCount = 0;
+	int skipScriptProcessing = 0;
+	int skipDiskWrite = 0;
+	int desiredServerId = 0;
+	int numItems = 0;
 
 	int id;
 	if (infile.ScanLine("%i", &id) != 1) goto error;
@@ -690,10 +817,19 @@ bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& i
 	if (!infile.ReadLine(buf, sizeof(buf))) goto error;
 	nzbInfo->SetDupeKey(buf);
 
-	int dupeMode, dupeScore;
-	if (infile.ScanLine("%i,%i", &dupeMode, &dupeScore) != 2) goto error;
+	int dupeMode, dupeScore, dupeHint;
+	dupeHint = 0; //clang requires initialization in a separate line (due to goto statements)
+	if (formatVersion >= 61)
+	{
+		if (infile.ScanLine("%i,%i,%i", &dupeMode, &dupeScore, &dupeHint) != 3) goto error;
+	}
+	else
+	{
+		if (infile.ScanLine("%i,%i", &dupeMode, &dupeScore) != 2) goto error;
+	}
 	nzbInfo->SetDupeMode((EDupeMode)dupeMode);
 	nzbInfo->SetDupeScore(dupeScore);
+	nzbInfo->SetDupeMode((EDupeMode)dupeHint);
 
 	if (formatVersion >= 48)
 	{
@@ -707,6 +843,7 @@ bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& i
 		nzbInfo->SetUnpackSec(unpackSec);
 	}
 
+	nzbInfo->GetCompletedFiles()->clear();
 	if (infile.ScanLine("%i", &fileCount) != 1) goto error;
 	for (int i = 0; i < fileCount; i++)
 	{
@@ -719,6 +856,8 @@ bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& i
 		int parFile = 0;
 		char* hash16k = nullptr;
 		char* parSetId = nullptr;
+		char filenameBuf[1024];
+		char origName[1024];
 
 		if (formatVersion >= 49)
 		{
@@ -756,16 +895,41 @@ bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& i
 			{
 				fileName++;
 			}
+			if (formatVersion >= 62)
+			{
+				if (!infile.ReadLine(filenameBuf, sizeof(filenameBuf))) goto error;
+				fileName = filenameBuf;
+				if (!infile.ReadLine(origName, sizeof(origName))) goto error;
+			}
 		}
 
-		nzbInfo->GetCompletedFiles()->emplace_back(id, fileName,
-			(CompletedFile::EStatus)status, crc, (bool)parFile,
-			Util::EmptyStr(hash16k) ? nullptr : hash16k,
-			Util::EmptyStr(parSetId) ? nullptr : parSetId);
+		nzbInfo->GetCompletedFiles()->emplace_back(
+			id, 
+			Util::EmptyStr(fileName) ? "" : fileName,
+			Util::EmptyStr(origName) ? "" : origName,
+			static_cast<CompletedFile::EStatus>(status), 
+			crc, 
+			static_cast<bool>(parFile),
+			Util::EmptyStr(hash16k) ? "" : hash16k,
+			Util::EmptyStr(parSetId) ? "" : parSetId);
+	}
+
+	nzbInfo->GetParameters()->clear();
+
+	numItems = infile.ScanLine("%i,%i,%i,%i", 
+		&parameterCount, 
+		&skipScriptProcessing, 
+		&skipDiskWrite,
+		&desiredServerId);
+
+	if (numItems < 1) goto error;
+	if (numItems == 4)
+	{
+		nzbInfo->SetSkipScriptProcessing(static_cast<bool>(skipScriptProcessing));
+		nzbInfo->SetSkipDiskWrite(static_cast<bool>(skipDiskWrite));
+		nzbInfo->SetDesiredServerId(desiredServerId);
 	}
 
-	int parameterCount;
-	if (infile.ScanLine("%i", &parameterCount) != 1) goto error;
 	for (int i = 0; i < parameterCount; i++)
 	{
 		if (!infile.ReadLine(buf, sizeof(buf))) goto error;
@@ -779,6 +943,7 @@ bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& i
 		}
 	}
 
+	nzbInfo->GetScriptStatuses()->clear();
 	int scriptCount;
 	if (infile.ScanLine("%i", &scriptCount) != 1) goto error;
 	for (int i = 0; i < scriptCount; i++)
@@ -808,6 +973,7 @@ bool DiskState::LoadNzbInfo(NzbInfo* nzbInfo, Servers* servers, StateDiskFile& i
 		}
 	}
 
+	nzbInfo->GetFileList()->clear();
 	if (infile.ScanLine("%i", &fileCount) != 1) goto error;
 	for (int i = 0; i < fileCount; i++)
 	{
@@ -900,6 +1066,8 @@ bool DiskState::SaveFileInfo(FileInfo* fileInfo, StateDiskFile& outfile, bool ar
 {
 	outfile.PrintLine("%s", fileInfo->GetSubject());
 	outfile.PrintLine("%s", fileInfo->GetFilename());
+	outfile.PrintLine("%s", fileInfo->GetOrigname() ? fileInfo->GetOrigname() : "");
+	outfile.PrintLine("%s", fileInfo->GetOutputFilename().c_str());
 
 	outfile.PrintLine("%i,%i", (int)fileInfo->GetFilenameConfirmed(), (int)fileInfo->GetTime());
 
@@ -963,6 +1131,21 @@ bool DiskState::LoadFileInfo(FileInfo* fileInfo, StateDiskFile& infile, int form
 	if (!infile.ReadLine(buf, sizeof(buf))) goto error;
 	if (fileSummary) fileInfo->SetFilename(buf);
 
+	if (formatVersion >= 6)
+	{
+		if (!infile.ReadLine(buf, sizeof(buf))) goto error;
+		if (fileSummary) fileInfo->SetOrigname(Util::EmptyStr(buf) ? nullptr : buf);
+	}
+
+	if (formatVersion >= 7)
+	{
+		if (!infile.ReadLine(buf, sizeof(buf))) goto error;
+		if (!Util::EmptyStr(buf))
+		{
+			fileInfo->SetOutputFilename(buf);
+		}
+	}
+
 	if (formatVersion >= 5)
 	{
 		int time, filenameConfirmed;
@@ -1055,7 +1238,8 @@ bool DiskState::SaveFileState(FileInfo* fileInfo, StateDiskFile& outfile, bool c
 	outfile.PrintLine("%u,%u,%u,%u,%u,%u", High1, Low1, High2, Low2, High3, Low3);
 
 	outfile.PrintLine("%s", fileInfo->GetFilename());
-	outfile.PrintLine("%s", fileInfo->GetHash16k());
+	outfile.PrintLine("%s", fileInfo->GetHash16k() ? fileInfo->GetHash16k() : "");
+	outfile.PrintLine("%s", fileInfo->GetParSetId() ? fileInfo->GetParSetId() : "");
 	outfile.PrintLine("%i", (int)fileInfo->GetParFile());
 
 	SaveServerStats(fileInfo->GetServerStats(), outfile);
@@ -1063,8 +1247,12 @@ bool DiskState::SaveFileState(FileInfo* fileInfo, StateDiskFile& outfile, bool c
 	outfile.PrintLine("%i", (int)fileInfo->GetArticles()->size());
 	for (ArticleInfo* articleInfo : fileInfo->GetArticles())
 	{
-		outfile.PrintLine("%i,%u,%i,%u", (int)articleInfo->GetStatus(), (uint32)articleInfo->GetSegmentOffset(),
-			articleInfo->GetSegmentSize(), (uint32)articleInfo->GetCrc());
+		outfile.PrintLine("%i,%" PRIi64 ",%i,%u", 
+			(int)articleInfo->GetStatus(), 
+			articleInfo->GetSegmentOffset(),
+			articleInfo->GetSegmentSize(), 
+			articleInfo->GetCrc()
+		);
 	}
 
 	outfile.Close();
@@ -1114,6 +1302,11 @@ bool DiskState::LoadFileState(FileInfo* fileInfo, Servers* servers, StateDiskFil
 	{
 		if (!infile.ReadLine(buf, sizeof(buf))) goto error;
 		fileInfo->SetHash16k(*buf ? buf : nullptr);
+		if (formatVersion >= 6)
+		{
+			if (!infile.ReadLine(buf, sizeof(buf))) goto error;
+			fileInfo->SetParSetId(*buf ? buf : nullptr);
+		}
 		int parFile = 0;
 		if (infile.ScanLine("%i", &parFile) != 1) goto error;
 		fileInfo->SetParFile((bool)parFile);
@@ -1138,9 +1331,10 @@ bool DiskState::LoadFileState(FileInfo* fileInfo, Servers* servers, StateDiskFil
 
 		if (formatVersion >= 2)
 		{
-			uint32 segmentOffset, crc;
+			int64 segmentOffset;
+			uint32 crc;
 			int segmentSize;
-			if (infile.ScanLine("%i,%u,%i,%u", &statusInt, &segmentOffset, &segmentSize, &crc) != 4) goto error;
+			if (infile.ScanLine("%i,%" PRIi64 ",%i,%u", &statusInt, &segmentOffset, &segmentSize, &crc) != 4) goto error;
 			pa->SetSegmentOffset(segmentOffset);
 			pa->SetSegmentSize(segmentSize);
 			pa->SetCrc(crc);
@@ -1451,7 +1645,8 @@ void DiskState::CleanupQueueDir(DownloadQueue* downloadQueue)
 
 	for (HistoryInfo* historyInfo : downloadQueue->GetHistory())
 	{
-		if (historyInfo->GetKind() == HistoryInfo::hkNzb)
+		if (historyInfo->GetKind() == HistoryInfo::hkNzb ||
+			historyInfo->GetKind() == HistoryInfo::hkUrl)
 		{
 			NzbInfo* nzbInfo = historyInfo->GetNzbInfo();
 			nzbIdList.push_back(nzbInfo->GetId());
@@ -2137,16 +2332,25 @@ bool DiskState::SaveVolumeStat(ServerVolumes* serverVolumes, StateDiskFile& outf
 	outfile.PrintLine("%i", (int)serverVolumes->size());
 	for (ServerVolume& serverVolume : serverVolumes)
 	{
-		outfile.PrintLine("%i,%i,%i", serverVolume.GetFirstDay(), (int)serverVolume.GetDataTime(), (int)serverVolume.GetCustomTime());
+		outfile.PrintLine("%i,%i,%i,%i", 
+			serverVolume.GetFirstDay(), 
+			static_cast<int>(serverVolume.GetDataTime()), 
+			static_cast<int>(serverVolume.GetCustomTime()),
+			static_cast<int>(serverVolume.GetCountersResetTime())
+		);
 
 		uint32 High1, Low1, High2, Low2;
 		Util::SplitInt64(serverVolume.GetTotalBytes(), &High1, &Low1);
 		Util::SplitInt64(serverVolume.GetCustomBytes(), &High2, &Low2);
 		outfile.PrintLine("%u,%u,%u,%u", High1, Low1, High2, Low2);
 
-		ServerVolume::VolumeArray* VolumeArrays[] = { serverVolume.BytesPerSeconds(),
-			serverVolume.BytesPerMinutes(), serverVolume.BytesPerHours(), serverVolume.BytesPerDays() };
-		for (int i=0; i < 4; i++)
+		ServerVolume::VolumeArray* VolumeArrays[] = { 
+			serverVolume.BytesPerSeconds(),
+			serverVolume.BytesPerMinutes(), 
+			serverVolume.BytesPerHours(), 
+			serverVolume.BytesPerDays() 
+		};
+		for (int i = 0; i < 4; ++i)
 		{
 			ServerVolume::VolumeArray* volumeArray = VolumeArrays[i];
 
@@ -2157,6 +2361,13 @@ bool DiskState::SaveVolumeStat(ServerVolumes* serverVolumes, StateDiskFile& outf
 				outfile.PrintLine("%u,%u", High1, Low1);
 			}
 		}
+		const auto& articlesPerDays = serverVolume.GetArticlesPerDays();
+		outfile.PrintLine("%u", Util::SafeIntCast<size_t, uint32>(articlesPerDays.size()));
+
+		for (ServerVolume::Articles articles : articlesPerDays)
+		{
+			outfile.PrintLine("%u,%u", articles.failed, articles.success);
+		}
 	}
 
 	return true;
@@ -2187,13 +2398,30 @@ bool DiskState::LoadVolumeStat(Servers* servers, ServerVolumes* serverVolumes, S
 			}
 		}
 
-		int firstDay, dataTime, customTime;
+		int firstDay, dataTime, customTime, countersResetTime;
 		uint32 High1, Low1, High2 = 0, Low2 = 0;
-		if (formatVersion >= 3)
+		if (formatVersion >= 4)
+		{
+			if (infile.ScanLine("%i,%i,%i,%i", &firstDay, &dataTime,&customTime, &countersResetTime) != 4) goto error;
+			if (infile.ScanLine("%u,%u,%u,%u", &High1, &Low1, &High2, &Low2) != 4) goto error;
+
+			if (serverVolume)
+			{
+				serverVolume->SetCustomTime(static_cast<time_t>(customTime));
+				serverVolume->SetCountersResetTime(static_cast<time_t>(countersResetTime));
+			}
+		}
+		else if (formatVersion == 3)
 		{
 			if (infile.ScanLine("%i,%i,%i", &firstDay, &dataTime,&customTime) != 3) goto error;
 			if (infile.ScanLine("%u,%u,%u,%u", &High1, &Low1, &High2, &Low2) != 4) goto error;
-			if (serverVolume) serverVolume->SetCustomTime((time_t)customTime);
+
+			if (serverVolume)
+			{
+				serverVolume->SetCustomTime(static_cast<time_t>(customTime));
+				time_t firstDayTime = static_cast<time_t>(firstDay * 86400);
+				serverVolume->SetCountersResetTime(firstDayTime);
+			}
 		}
 		else
 		{
@@ -2205,7 +2433,8 @@ bool DiskState::LoadVolumeStat(Servers* servers, ServerVolumes* serverVolumes, S
 		if (serverVolume) serverVolume->SetTotalBytes(Util::JoinInt64(High1, Low1));
 		if (serverVolume) serverVolume->SetCustomBytes(Util::JoinInt64(High2, Low2));
 
-		ServerVolume::VolumeArray* VolumeArrays[] = { serverVolume ? serverVolume->BytesPerSeconds() : nullptr,
+		ServerVolume::VolumeArray* VolumeArrays[] = { 
+			serverVolume ? serverVolume->BytesPerSeconds() : nullptr,
 			serverVolume ? serverVolume->BytesPerMinutes() : nullptr,
 			serverVolume ? serverVolume->BytesPerHours() : nullptr,
 			serverVolume ? serverVolume->BytesPerDays() : nullptr };
@@ -2223,6 +2452,24 @@ bool DiskState::LoadVolumeStat(Servers* servers, ServerVolumes* serverVolumes, S
 				if (volumeArray) (*volumeArray)[j] = Util::JoinInt64(High1, Low1);
 			}
 		}
+
+		if (formatVersion >= 4)
+		{
+			uint32 arrSize;
+			if (infile.ScanLine("%u", &arrSize) != 1) goto error;
+
+			ServerVolume::ArticlesArray articlesArr;
+			articlesArr.reserve(arrSize);
+
+			ServerVolume::Articles articles;
+			while (arrSize--)
+			{
+				if (infile.ScanLine("%u,%u", &articles.failed, &articles.success) != 2) goto error;
+				articlesArr.push_back(articles);
+			}
+
+			if (serverVolume) serverVolume->SetArticlesPerDays(std::move(articlesArr));
+		}
 	}
 
 	return true;
