@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ public:
 	void Redownload(DownloadQueue* downloadQueue, HistoryInfo* historyInfo);
 
 protected:
-	virtual int ServiceInterval() { return 600000; }
+	virtual int ServiceInterval() { return 60 * 60; }
 	virtual void ServiceWork();
 
 private:
