@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@users.sourceforge.net>
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 #include "nzbget.h"
 #include "QueueCoordinator.h"
 #include "Options.h"
+#include "WorkState.h"
 #include "ServerPool.h"
 #include "ArticleDownloader.h"
 #include "ArticleWriter.h"
@@ -30,6 +32,7 @@
 #include "FileSystem.h"
 #include "Decoder.h"
 #include "StatMeter.h"
+#include "Deobfuscation.h"
 
 bool QueueCoordinator::CoordinatorDownloadQueue::EditEntry(
 	int ID, EEditAction action, const char* args)
@@ -71,6 +74,18 @@ void QueueCoordinator::CoordinatorDownloadQueue::Save()
 
 	m_wantSave = false;
 	m_historyChanged = false;
+
+	// queue has changed, time to wake up if in standby
+	m_owner->WakeUp();
+}
+
+void QueueCoordinator::CoordinatorDownloadQueue::SaveChanged()
+{
+	if (g_Options->GetServerMode())
+	{
+		g_DiskState->SaveDownloadProgress(this);
+		m_stateChanged = true;
+	}
 }
 
 QueueCoordinator::QueueCoordinator()
@@ -78,6 +93,7 @@ QueueCoordinator::QueueCoordinator()
 	debug("Creating QueueCoordinator");
 
 	CoordinatorDownloadQueue::Init(&m_downloadQueue);
+	g_WorkState->Attach(this);
 }
 
 QueueCoordinator::~QueueCoordinator()
@@ -177,43 +193,44 @@ void QueueCoordinator::Run()
 	AdjustDownloadsLimit();
 	bool wasStandBy = true;
 	bool articeDownloadsRunning = false;
-	int resetCounter = 0;
+	time_t lastReset = 0;
 	g_StatMeter->IntervalCheck();
-
+	int waitInterval = 100;
 	while (!IsStopped())
 	{
 		bool downloadsChecked = false;
 		bool downloadStarted = false;
-		NntpConnection* connection = g_ServerPool->GetConnection(0, nullptr, nullptr);
-		if (connection)
+		FileInfo* fileInfo;
+		ArticleInfo* articleInfo;
 		{
-			// start download for next article
-			FileInfo* fileInfo;
-			ArticleInfo* articleInfo;
-			bool freeConnection = false;
+			GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 
+			bool hasMoreArticles = GetNextArticle(downloadQueue, fileInfo, articleInfo);
+			articeDownloadsRunning = !m_activeDownloads.empty();
+			m_hasMoreJobs = hasMoreArticles || articeDownloadsRunning;
+			bool canProceed = !IsStopped() && Util::SafeIntCast<size_t, int>(m_activeDownloads.size()) < m_downloadsLimit;
+
+			if (hasMoreArticles && canProceed && (!g_WorkState->GetTempPauseDownload() || fileInfo->GetExtraPriority()))
 			{
-				GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
-				bool hasMoreArticles = GetNextArticle(downloadQueue, fileInfo, articleInfo);
-				articeDownloadsRunning = !m_activeDownloads.empty();
-				downloadsChecked = true;
-				m_hasMoreJobs = hasMoreArticles || articeDownloadsRunning;
-				if (hasMoreArticles && !IsStopped() && (int)m_activeDownloads.size() < m_downloadsLimit &&
-					(!g_Options->GetTempPauseDownload() || fileInfo->GetExtraPriority()))
+				NntpConnection* connection = nullptr;
+				NewsServer* desiredServer = g_ServerPool->GetServerById(fileInfo->GetNzbInfo()->GetDesiredServerId());
+
+				if (desiredServer)
 				{
-					StartArticleDownload(fileInfo, articleInfo, connection);
-					articeDownloadsRunning = true;
-					downloadStarted = true;
+					connection = g_ServerPool->GetConnection(desiredServer->GetLevel(), desiredServer, nullptr);
 				}
 				else
 				{
-					freeConnection = true;
+					connection = g_ServerPool->GetConnection(0, nullptr, nullptr);
+				}
+				 
+				if (connection)
+				{
+					downloadsChecked = true;
+					articeDownloadsRunning = true;
+					downloadStarted = true;
+					StartArticleDownload(fileInfo, articleInfo, connection);
 				}
-			}
-
-			if (freeConnection)
-			{
-				g_ServerPool->FreeConnection(connection, false);
 			}
 		}
 
@@ -227,6 +244,7 @@ void QueueCoordinator::Run()
 		if (standBy != wasStandBy)
 		{
 			g_StatMeter->EnterLeaveStandBy(standBy);
+			g_WorkState->SetDownloading(!standBy);
 			wasStandBy = standBy;
 			if (standBy)
 			{
@@ -235,18 +253,23 @@ void QueueCoordinator::Run()
 		}
 
 		// sleep longer in StandBy
-		int sleepInterval = downloadStarted ? 0 : standBy ? 100 : 5;
-		usleep(sleepInterval * 1000);
-
-		if (!standBy)
+		if (standBy)
 		{
+			Guard guard(m_waitMutex);
+			// sleeping max. 2 seconds; can't sleep much longer because we can't rely on
+			// notifications from 'WorkState' and we also have periodical work to do here
+			waitInterval = std::min(waitInterval * 2, 2000);
+			m_waitCond.WaitFor(m_waitMutex, waitInterval, [&]{ return m_hasMoreJobs || IsStopped(); });
+		}
+		else
+		{
+			int sleepInterval = downloadStarted ? 0 : 5;
+			Util::Sleep(sleepInterval);
 			g_StatMeter->AddSpeedReading(0);
+			waitInterval = 100;
 		}
 
-		Util::SetStandByMode(standBy);
-
-		resetCounter += sleepInterval;
-		if (resetCounter >= 1000)
+		if (lastReset != Util::CurrentTime())
 		{
 			// this code should not be called too often, once per second is OK
 			g_ServerPool->CloseUnusedConnections();
@@ -255,20 +278,31 @@ void QueueCoordinator::Run()
 			{
 				SaveAllPartialState();
 			}
-			resetCounter = 0;
 			g_StatMeter->IntervalCheck();
 			g_Log->IntervalCheck();
 			AdjustDownloadsLimit();
+			Util::SetStandByMode(standBy);
+			lastReset = Util::CurrentTime();
 		}
 	}
 
 	WaitJobs();
 	SaveAllPartialState();
+	SaveQueueIfChanged();
 	SaveAllFileState();
 
 	debug("Exiting QueueCoordinator-loop");
 }
 
+void QueueCoordinator::WakeUp()
+{
+	debug("Waking up QueueCoordinator");
+	// Resume Run()
+	Guard guard(m_waitMutex);
+	m_hasMoreJobs = true;
+	m_waitCond.NotifyAll();
+}
+
 void QueueCoordinator::WaitJobs()
 {
 	// waiting for downloads
@@ -283,7 +317,7 @@ void QueueCoordinator::WaitJobs()
 				break;
 			}
 		}
-		usleep(100 * 1000);
+		Util::Sleep(100);
 		ResetHangingDownloads();
 	}
 
@@ -419,7 +453,7 @@ void QueueCoordinator::CheckDupeFileInfos(NzbInfo* nzbInfo)
 			{
 				// If more than two files have same filename we don't filter them out since that
 				// naming might be intentional and correct filenames must be read from article bodies.
-				int dupeCount = std::count_if(nzbInfo->GetFileList()->begin(), nzbInfo->GetFileList()->end(),
+				int dupeCount = (int)std::count_if(nzbInfo->GetFileList()->begin(), nzbInfo->GetFileList()->end(),
 					[fileInfo2](std::unique_ptr<FileInfo>& fileInfo3)
 					{
 						return !strcmp(fileInfo3->GetFilename(), fileInfo2->GetFilename());
@@ -455,12 +489,18 @@ void QueueCoordinator::Stop()
 	Thread::Stop();
 
 	debug("Stopping ArticleDownloads");
-	GuardedDownloadQueue guard = DownloadQueue::Guard();
-	for (ArticleDownloader* articleDownloader : m_activeDownloads)
 	{
-		articleDownloader->Stop();
+		GuardedDownloadQueue guard = DownloadQueue::Guard();
+		for (ArticleDownloader* articleDownloader : m_activeDownloads)
+		{
+			articleDownloader->Stop();
+		}
 	}
 	debug("ArticleDownloads are notified");
+
+	// Resume Run() to exit it
+	Guard guard(m_waitMutex);
+	m_waitCond.NotifyAll();
 }
 
 /*
@@ -474,8 +514,6 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 
 	// special case: if the file has ExtraPriority-flag set, it has the highest priority.
 
-	//debug("QueueCoordinator::GetNextArticle()");
-
 	bool ok = false;
 
 	RawFileList checkedFiles;
@@ -495,7 +533,7 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 			bool nzbPaused = nzbInfo->GetFileList()->size() - nzbInfo->GetPausedFileCount() <= 0;
 
 			if ((!fileInfo || nzbHigherPriority) && !nzbPaused &&
-				(!(g_Options->GetPauseDownload() || g_Options->GetQuotaReached()) || nzbInfo->GetForcePriority()))
+				(!(g_WorkState->GetPauseDownload() || g_WorkState->GetQuotaReached()) || nzbInfo->GetForcePriority()))
 			{
 				for (FileInfo* fileInfo1 : nzbInfo->GetFileList())
 				{
@@ -619,11 +657,18 @@ void QueueCoordinator::StartArticleDownload(FileInfo* fileInfo, ArticleInfo* art
 	articleDownloader->Start();
 }
 
-void QueueCoordinator::Update(Subject* Caller, void* Aspect)
+void QueueCoordinator::Update(Subject* caller, void* aspect)
 {
+	if (caller == g_WorkState)
+	{
+		debug("Notification from WorkState received");
+		WakeUp();
+		return;
+	}
+
 	debug("Notification from ArticleDownloader received");
 
-	ArticleDownloader* articleDownloader = (ArticleDownloader*)Caller;
+	ArticleDownloader* articleDownloader = (ArticleDownloader*)caller;
 	if ((articleDownloader->GetStatus() == ArticleDownloader::adFinished) ||
 		(articleDownloader->GetStatus() == ArticleDownloader::adFailed) ||
 		(articleDownloader->GetStatus() == ArticleDownloader::adRetry))
@@ -698,7 +743,7 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* articleDownloader)
 			// if the name from article seems to be obfuscated
 			bool useFilenameFromArticle = g_Options->GetFileNaming() == Options::nfArticle ||
 				(g_Options->GetFileNaming() == Options::nfAuto &&
-				 !Util::AlphaNum(articleDownloader->GetArticleFilename()) &&
+				 !Deobfuscation::IsExcessivelyObfuscated(articleDownloader->GetArticleFilename()) &&
 				 !nzbInfo->GetManyDupeFiles());
 			if (useFilenameFromArticle)
 			{
@@ -768,7 +813,8 @@ void QueueCoordinator::DeleteDownloader(DownloadQueue* downloadQueue,
 	if (deleteFileObj)
 	{
 		DeleteFileInfo(downloadQueue, fileInfo, fileCompleted);
-		downloadQueue->Save();
+		nzbInfo->SetChanged(true);
+		downloadQueue->SaveChanged();
 	}
 }
 
@@ -776,7 +822,7 @@ void QueueCoordinator::DeleteFileInfo(DownloadQueue* downloadQueue, FileInfo* fi
 {
 	while (g_ArticleCache->FileBusy(fileInfo))
 	{
-		usleep(5*1000);
+		Util::Sleep(5);
 	}
 
 	NzbInfo* nzbInfo = fileInfo->GetNzbInfo();
@@ -816,13 +862,21 @@ void QueueCoordinator::DeleteFileInfo(DownloadQueue* downloadQueue, FileInfo* fi
 
 	if (completed || parking)
 	{
+		const std::string& outputFilename = fileInfo->GetOutputFilename();
+		std::string filename = (completed && !outputFilename.empty())
+			? FileSystem::BaseFileName(outputFilename.c_str())
+			: (fileInfo->GetFilename() ? fileInfo->GetFilename() : "");
+
 		fileInfo->GetNzbInfo()->GetCompletedFiles()->emplace_back(
 			fileInfo->GetId(),
-			completed && fileInfo->GetOutputFilename() ?
-			FileSystem::BaseFileName(fileInfo->GetOutputFilename()) : fileInfo->GetFilename(),
+			std::move(filename),
+			fileInfo->GetOrigname() ? fileInfo->GetOrigname() : "", 
 			fileStatus,
 			fileStatus == CompletedFile::cfSuccess ? fileInfo->GetCrc() : 0,
-			fileInfo->GetParFile(), fileInfo->GetHash16k(), fileInfo->GetParSetId());
+			fileInfo->GetParFile(), 
+			fileInfo->GetHash16k() ? fileInfo->GetHash16k() : "", 
+			fileInfo->GetParSetId() ? fileInfo->GetParSetId() : ""
+		);
 	}
 
 	if (g_Options->GetDirectRename())
@@ -860,39 +914,52 @@ void QueueCoordinator::DiscardTempFiles(FileInfo* fileInfo)
 		}
 	}
 
-	if (g_Options->GetDirectWrite() && fileInfo->GetOutputFilename() && !fileInfo->GetForceDirectWrite())
+	const std::string& outputFilename = fileInfo->GetOutputFilename();
+	if (g_Options->GetDirectWrite() && !outputFilename.empty() && !fileInfo->GetForceDirectWrite())
 	{
-		FileSystem::DeleteFile(fileInfo->GetOutputFilename());
+		FileSystem::DeleteFile(outputFilename.c_str());
 	}
 }
 
-void QueueCoordinator::SaveAllPartialState()
+void QueueCoordinator::SaveQueueIfChanged()
 {
 	if (!g_Options->GetServerMode())
 	{
 		return;
 	}
 
-	bool hasUnsavedData = false;
+	bool hasChanges = false;
 	GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 	for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
 	{
-		if (g_Options->GetContinuePartial())
-		{
-			for (FileInfo* fileInfo : nzbInfo->GetFileList())
-			{
-				SavePartialState(fileInfo);
-			}
-		}
-		hasUnsavedData |= nzbInfo->GetChanged();
+		hasChanges |= nzbInfo->GetChanged();
 	}
 
-	if (hasUnsavedData)
+	if (hasChanges)
 	{
 		downloadQueue->Save();
 	}
 }
 
+void QueueCoordinator::SaveAllPartialState()
+{
+	if (!g_Options->GetServerMode() || !g_Options->GetContinuePartial())
+	{
+		return;
+	}
+
+	GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
+	for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
+	{
+		for (FileInfo* fileInfo : nzbInfo->GetFileList())
+		{
+			SavePartialState(fileInfo);
+		}
+	}
+
+	downloadQueue->SaveChanged();
+}
+
 void QueueCoordinator::SavePartialState(FileInfo* fileInfo)
 {
 	if (fileInfo->GetPartialChanged())
@@ -978,9 +1045,9 @@ void QueueCoordinator::LogDebugInfo()
 	downloadQueue->CalcRemainingSize(&remaining, &remainingForced);
 	info("     Remaining: %.1f MB, Forced: %.1f MB", remaining / 1024.0 / 1024.0, remainingForced / 1024.0 / 1024.0);
 	info("     Download: %s, Post-process: %s, Scan: %s",
-		 (g_Options->GetPauseDownload() ? "paused" : g_Options->GetTempPauseDownload() ? "temp-paused" : "active"),
-		 (g_Options->GetPausePostProcess() ? "paused" : "active"),
-		 (g_Options->GetPauseScan() ? "paused" : "active"));
+		 (g_WorkState->GetPauseDownload() ? "paused" : g_WorkState->GetTempPauseDownload() ? "temp-paused" : "active"),
+		 (g_WorkState->GetPausePostProcess() ? "paused" : "active"),
+		 (g_WorkState->GetPauseScan() ? "paused" : "active"));
 
 	info("   ---------- QueueCoordinator");
 	info("    Active Downloads: %i, Limit: %i", (int)m_activeDownloads.size(), m_downloadsLimit);
@@ -1315,7 +1382,8 @@ void QueueCoordinator::DirectRenameCompleted(DownloadQueue* downloadQueue, NzbIn
 		downloadQueue->EditEntry(nzbInfo->GetId(), DownloadQueue::eaGroupSortFiles, nullptr);
 	}
 
-	downloadQueue->Save();
+	nzbInfo->SetChanged(true);
+	downloadQueue->SaveChanged();
 
 	DownloadQueue::Aspect namedAspect = { DownloadQueue::eaNzbNamed, downloadQueue, nzbInfo, nullptr };
 	downloadQueue->Notify(&namedAspect);
