@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,13 +15,14 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #include "nzbget.h"
 #include "Scanner.h"
 #include "Options.h"
+#include "WorkState.h"
 #include "Log.h"
 #include "QueueCoordinator.h"
 #include "HistoryCoordinator.h"
@@ -73,60 +75,72 @@ void Scanner::QueueData::SetNzbId(int nzbId)
 
 void Scanner::InitOptions()
 {
-	m_nzbDirInterval = g_Options->GetNzbDirInterval() * 1000;
+	m_nzbDirInterval = 1;
 	m_scanScript = ScanScriptController::HasScripts();
 }
 
+int Scanner::ServiceInterval()
+{
+	return m_requestedNzbDirScan ? Service::Now :
+		g_Options->GetNzbDirInterval() <= 0 ? Service::Sleep :
+		// g_Options->GetPauseScan() ? Service::Sleep :   // for that to work we need to react on changing of pause-state
+		m_nzbDirInterval;
+}
+
 void Scanner::ServiceWork()
 {
+	debug("Scanner service work");
+
 	if (!DownloadQueue::IsLoaded())
 	{
 		return;
 	}
 
-	Guard guard(m_scanMutex);
+	m_nzbDirInterval = g_Options->GetNzbDirInterval();
 
-	if (m_requestedNzbDirScan ||
-		(!g_Options->GetPauseScan() && g_Options->GetNzbDirInterval() > 0 &&
-		 m_nzbDirInterval >= g_Options->GetNzbDirInterval() * 1000))
+	if (g_WorkState->GetPauseScan() && !m_requestedNzbDirScan)
 	{
-		// check nzbdir every g_pOptions->GetNzbDirInterval() seconds or if requested
-		bool checkStat = !m_requestedNzbDirScan;
-		m_requestedNzbDirScan = false;
-		m_scanning = true;
-		CheckIncomingNzbs(g_Options->GetNzbDir(), "", checkStat);
-		if (!checkStat && m_scanScript)
-		{
-			// if immediate scan requested, we need second scan to process files extracted by scan-scripts
-			CheckIncomingNzbs(g_Options->GetNzbDir(), "", checkStat);
-		}
-		m_scanning = false;
-		m_nzbDirInterval = 0;
-
-		// if NzbDirFileAge is less than NzbDirInterval (that can happen if NzbDirInterval
-		// is set for rare scans like once per hour) we make 4 scans:
-		//   - one additional scan is neccessary to check sizes of detected files;
-		//   - another scan is required to check files which were extracted by scan-scripts;
-		//   - third scan is needed to check sizes of extracted files.
-		if (g_Options->GetNzbDirInterval() > 0 && g_Options->GetNzbDirFileAge() < g_Options->GetNzbDirInterval())
-		{
-			int maxPass = m_scanScript ? 3 : 1;
-			if (m_pass < maxPass)
-			{
-				// scheduling another scan of incoming directory in NzbDirFileAge seconds.
-				m_nzbDirInterval = (g_Options->GetNzbDirInterval() - g_Options->GetNzbDirFileAge()) * 1000;
-				m_pass++;
-			}
-			else
-			{
-				m_pass = 0;
-			}
-		}
-
-		DropOldFiles();
-		m_queueList.clear();
+		return;
 	}
-	m_nzbDirInterval += 200;
+
+	debug("Scanner service work: doing work");
+
+	std::lock_guard<std::mutex> guard{m_scanMutex};
+
+	// check nzbdir every g_pOptions->GetNzbDirInterval() seconds or if requested
+	bool checkStat = !m_requestedNzbDirScan;
+	m_requestedNzbDirScan = false;
+	m_scanning = true;
+	CheckIncomingNzbs(g_Options->GetNzbDir(), "", checkStat);
+	if (!checkStat && m_scanScript)
+	{
+		// if immediate scan requested, we need second scan to process files extracted by scan-scripts
+		CheckIncomingNzbs(g_Options->GetNzbDir(), "", checkStat);
+	}
+	m_scanning = false;
+
+	// if NzbDirFileAge is less than NzbDirInterval (that can happen if NzbDirInterval
+	// is set for rare scans like once per hour) we make 4 scans:
+	//   - one additional scan is neccessary to check sizes of detected files;
+	//   - another scan is required to check files which were extracted by scan-scripts;
+	//   - third scan is needed to check sizes of extracted files.
+	if (g_Options->GetNzbDirInterval() > 0 && g_Options->GetNzbDirFileAge() < g_Options->GetNzbDirInterval())
+	{
+		int maxPass = m_scanScript ? 3 : 1;
+		if (m_pass < maxPass)
+		{
+			// scheduling another scan of incoming directory in NzbDirFileAge seconds.
+			m_nzbDirInterval = g_Options->GetNzbDirFileAge();
+			m_pass++;
+		}
+		else
+		{
+			m_pass = 0;
+		}
+	}
+
+	DropOldFiles();
+	m_queueList.clear();
 }
 
 /**
@@ -270,7 +284,7 @@ void Scanner::ProcessIncomingFile(const char* directory, const char* baseFilenam
 	EDupeMode dupeMode = dmScore;
 	EAddStatus addStatus = asSkipped;
 	QueueData* queueData = nullptr;
-	NzbInfo* urlInfo = nullptr;
+	NzbInfo* nzbInfo = nullptr;
 	int nzbId = 0;
 
 	for (QueueData& queueData1 : m_queueList)
@@ -287,7 +301,7 @@ void Scanner::ProcessIncomingFile(const char* directory, const char* baseFilenam
 			addTop = queueData->GetAddTop();
 			addPaused = queueData->GetAddPaused();
 			parameters.CopyFrom(queueData->GetParameters());
-			urlInfo = queueData->GetUrlInfo();
+			nzbInfo = queueData->GetUrlInfo();
 		}
 	}
 
@@ -298,7 +312,7 @@ void Scanner::ProcessIncomingFile(const char* directory, const char* baseFilenam
 	if (m_scanScript && strcasecmp(extension, ".nzb_processed"))
 	{
 		ScanScriptController::ExecuteScripts(fullFilename,
-			urlInfo ? urlInfo->GetUrl() : "", directory,
+			nzbInfo, directory,
 			&nzbName, &nzbCategory, &priority, &parameters, &addTop,
 			&addPaused, &dupeKey, &dupeScore, &dupeMode);
 		exists = FileSystem::FileExists(fullFilename);
@@ -321,7 +335,7 @@ void Scanner::ProcessIncomingFile(const char* directory, const char* baseFilenam
 		if (renameOK)
 		{
 			bool added = AddFileToQueue(renamedName, nzbName, nzbCategory, priority,
-				dupeKey, dupeScore, dupeMode, &parameters, addTop, addPaused, urlInfo, &nzbId);
+				dupeKey, dupeScore, dupeMode, &parameters, addTop, addPaused, nzbInfo, &nzbId);
 			addStatus = added ? asSuccess : asFailed;
 		}
 		else
@@ -334,7 +348,7 @@ void Scanner::ProcessIncomingFile(const char* directory, const char* baseFilenam
 	else if (exists && !strcasecmp(extension, ".nzb"))
 	{
 		bool added = AddFileToQueue(fullFilename, nzbName, nzbCategory, priority,
-			dupeKey, dupeScore, dupeMode, &parameters, addTop, addPaused, urlInfo, &nzbId);
+			dupeKey, dupeScore, dupeMode, &parameters, addTop, addPaused, nzbInfo, &nzbId);
 		addStatus = added ? asSuccess : asFailed;
 	}
 
@@ -365,13 +379,16 @@ void Scanner::InitPPParameters(const char* category, NzbParameterList* parameter
 
 	if (reset)
 	{
-		for (ScriptConfig::Script& script : g_ScriptConfig->GetScripts())
+		for (const auto script : g_ExtensionManager->GetExtensions())
 		{
-			parameters->SetParameter(BString<1024>("%s:", script.GetName()), nullptr);
+			parameters->SetParameter(BString<1024>("%s:", script->GetName()), nullptr);
 		}
 	}
 
-	parameters->SetParameter("*Unpack:", unpack ? "yes" : "no");
+	if (!parameters->Find("*Unpack:"))
+	{
+		parameters->SetParameter("*Unpack:", unpack ? "yes" : "no");
+	}
 
 	if (!Util::EmptyStr(extensions))
 	{
@@ -379,12 +396,14 @@ void Scanner::InitPPParameters(const char* category, NzbParameterList* parameter
 		Tokenizer tok(extensions, ",;");
 		while (const char* scriptName = tok.Next())
 		{
-			for (ScriptConfig::Script& script : g_ScriptConfig->GetScripts())
+			for (const auto script : g_ExtensionManager->GetExtensions())
 			{
-				if ((script.GetPostScript() || script.GetQueueScript()) &&
-					FileSystem::SameFilename(scriptName, script.GetName()))
+				BString<1024> paramName("%s:", scriptName);
+				if ((script->GetPostScript() || script->GetQueueScript()) &&
+					!parameters->Find(paramName) &&
+					strcmp(scriptName, script->GetName()) == 0)
 				{
-					parameters->SetParameter(BString<1024>("%s:", scriptName), "yes");
+					parameters->SetParameter(paramName, "yes");
 				}
 			}
 		}
@@ -433,11 +452,15 @@ bool Scanner::AddFileToQueue(const char* filename, const char* nzbName, const ch
 		nzbInfo->SetUrl(urlInfo->GetUrl());
 		nzbInfo->SetUrlStatus(urlInfo->GetUrlStatus());
 		nzbInfo->SetFeedId(urlInfo->GetFeedId());
+		nzbInfo->SetDupeHint(urlInfo->GetDupeHint());
+		nzbInfo->SetDesiredServerId(urlInfo->GetDesiredServerId());
+		nzbInfo->SetSkipScriptProcessing(urlInfo->GetSkipScriptProcessing());
+		nzbInfo->SetSkipDiskWrite(urlInfo->GetSkipDiskWrite());
 	}
 
-	if (nzbFile.GetPassword())
+	if (!nzbFile.GetPassword().empty())
 	{
-		nzbInfo->GetParameters()->SetParameter("*Unpack:Password", nzbFile.GetPassword());
+		nzbInfo->GetParameters()->SetParameter("*Unpack:Password", nzbFile.GetPassword().c_str());
 	}
 
 	nzbInfo->GetParameters()->CopyFrom(parameters);
@@ -453,7 +476,14 @@ bool Scanner::AddFileToQueue(const char* filename, const char* nzbName, const ch
 	{
 		addedNzb = g_QueueCoordinator->AddNzbFileToQueue(std::move(nzbInfo), std::move(urlInfo), addTop);
 	}
-	else if (!urlInfo)
+	else if (urlInfo)
+	{
+		for (Message& message : nzbInfo->GuardCachedMessages())
+		{
+			urlInfo->AddMessage(message.GetKind(), message.GetText(), false);
+		}
+	}
+	else
 	{
 		nzbInfo->SetDeleteStatus(NzbInfo::dsScan);
 		addedNzb = g_QueueCoordinator->AddNzbFileToQueue(std::move(nzbInfo), std::move(urlInfo), addTop);
@@ -470,19 +500,20 @@ bool Scanner::AddFileToQueue(const char* filename, const char* nzbName, const ch
 void Scanner::ScanNzbDir(bool syncMode)
 {
 	{
-		Guard guard(m_scanMutex);
+		std::lock_guard<std::mutex> guard{m_scanMutex};
 		m_scanning = true;
 		m_requestedNzbDirScan = true;
+		WakeUp();
 	}
 
 	while (syncMode && (m_scanning || m_requestedNzbDirScan))
 	{
-		usleep(100 * 1000);
+		Util::Sleep(100);
 	}
 }
 
 Scanner::EAddStatus Scanner::AddExternalFile(const char* nzbName, const char* category,
-	int priority, const char* dupeKey, int dupeScore,  EDupeMode dupeMode,
+	int priority, const char* dupeKey, int dupeScore, EDupeMode dupeMode,
 	NzbParameterList* parameters, bool addTop, bool addPaused, NzbInfo* urlInfo,
 	const char* fileName, const char* buffer, int bufSize, int* nzbId)
 {
@@ -551,7 +582,7 @@ Scanner::EAddStatus Scanner::AddExternalFile(const char* nzbName, const char* ca
 	EAddStatus addStatus;
 
 	{
-		Guard guard(m_scanMutex);
+		std::lock_guard<std::mutex> guard{m_scanMutex};
 
 		if (!FileSystem::MoveFile(tempFileName, scanFileName))
 		{
