@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2012-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2012-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "nzbget.h"
 #include "UrlCoordinator.h"
 #include "Options.h"
+#include "WorkState.h"
 #include "WebDownloader.h"
 #include "Util.h"
 #include "FileSystem.h"
@@ -60,6 +61,12 @@ void UrlDownloader::ProcessHeader(const char* line)
 	}
 }
 
+UrlCoordinator::UrlCoordinator()
+{
+	m_downloadQueueObserver.m_owner = this;
+	DownloadQueue::Guard()->Attach(&m_downloadQueueObserver);
+}
+
 UrlCoordinator::~UrlCoordinator()
 {
 	debug("Destroying UrlCoordinator");
@@ -79,41 +86,45 @@ void UrlCoordinator::Run()
 
 	while (!DownloadQueue::IsLoaded())
 	{
-		usleep(20 * 1000);
+		Util::Sleep(20);
 	}
 
-	int resetCounter = 0;
-
 	while (!IsStopped())
 	{
+		time_t lastReset = 0;
 		bool downloadStarted = false;
-		if (!g_Options->GetPauseDownload() || g_Options->GetUrlForce())
+
 		{
-			// start download for next URL
+			NzbInfo* nzbInfo = nullptr;
 			GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 			if ((int)m_activeDownloads.size() < g_Options->GetUrlConnections())
 			{
-				NzbInfo* nzbInfo = GetNextUrl(downloadQueue);
-				bool hasMoreUrls = nzbInfo != nullptr;
-				bool urlDownloadsRunning = !m_activeDownloads.empty();
-				m_hasMoreJobs = hasMoreUrls || urlDownloadsRunning;
-				if (hasMoreUrls && !IsStopped())
+				nzbInfo = GetNextUrl(downloadQueue);
+				if (nzbInfo && (!g_WorkState->GetPauseDownload() || g_Options->GetUrlForce()))
 				{
 					StartUrlDownload(nzbInfo);
 					downloadStarted = true;
 				}
 			}
+			m_hasMoreJobs = !m_activeDownloads.empty() || nzbInfo;
 		}
 
-		int sleepInterval = downloadStarted ? 0 : 100;
-		usleep(sleepInterval * 1000);
-
-		resetCounter += sleepInterval;
-		if (resetCounter >= 1000)
+		if (lastReset != Util::CurrentTime())
 		{
 			// this code should not be called too often, once per second is OK
 			ResetHangingDownloads();
-			resetCounter = 0;
+			lastReset = Util::CurrentTime();
+		}
+
+		if (!m_hasMoreJobs && !IsStopped())
+		{
+			Guard guard(m_waitMutex);
+			m_waitCond.Wait(m_waitMutex, [&] { return m_hasMoreJobs || IsStopped(); });
+		}
+		else
+		{
+			int sleepInterval = downloadStarted ? 0 : 100;
+			Util::Sleep(sleepInterval);
 		}
 	}
 
@@ -136,7 +147,7 @@ void UrlCoordinator::WaitJobs()
 				break;
 			}
 		}
-		usleep(100 * 1000);
+		Util::Sleep(100);
 		ResetHangingDownloads();
 	}
 
@@ -148,18 +159,38 @@ void UrlCoordinator::Stop()
 	Thread::Stop();
 
 	debug("Stopping UrlDownloads");
-	GuardedDownloadQueue guard = DownloadQueue::Guard();
-	for (UrlDownloader* urlDownloader : m_activeDownloads)
 	{
-		urlDownloader->Stop();
+		GuardedDownloadQueue guard = DownloadQueue::Guard();
+		for (UrlDownloader* urlDownloader : m_activeDownloads)
+		{
+			urlDownloader->Stop();
+		}
 	}
 	debug("UrlDownloads are notified");
+
+	// Resume Run() to exit it
+	Guard guard(m_waitMutex);
+	m_waitCond.NotifyAll();
+}
+
+void UrlCoordinator::DownloadQueueUpdate(Subject* caller, void* aspect)
+{
+	debug("Notification from download queue received");
+
+	DownloadQueue::Aspect* queueAspect = (DownloadQueue::Aspect*)aspect;
+	if (queueAspect->action == DownloadQueue::eaUrlAdded ||
+		queueAspect->action == DownloadQueue::eaUrlReturned)
+	{
+		// Resume Run()
+		Guard guard(m_waitMutex);
+		m_hasMoreJobs = true;
+		m_waitCond.NotifyAll();
+	}
 }
 
 void UrlCoordinator::ResetHangingDownloads()
 {
-	const int timeout = g_Options->GetTerminateTimeout();
-	if (timeout == 0)
+	if (g_Options->GetUrlTimeout() == 0)
 	{
 		return;
 	}
@@ -167,32 +198,15 @@ void UrlCoordinator::ResetHangingDownloads()
 	GuardedDownloadQueue guard = DownloadQueue::Guard();
 	time_t tm = Util::CurrentTime();
 
-	m_activeDownloads.erase(std::remove_if(m_activeDownloads.begin(), m_activeDownloads.end(),
-		[timeout, tm](UrlDownloader* urlDownloader)
+	for (UrlDownloader* urlDownloader: m_activeDownloads)
+	{
+		if (tm - urlDownloader->GetLastUpdateTime() > g_Options->GetUrlTimeout() + 10 &&
+			urlDownloader->GetStatus() == UrlDownloader::adRunning)
 		{
-			if (tm - urlDownloader->GetLastUpdateTime() > timeout &&
-				urlDownloader->GetStatus() == UrlDownloader::adRunning)
-			{
-				NzbInfo* nzbInfo = urlDownloader->GetNzbInfo();
-				debug("Terminating hanging download %s", urlDownloader->GetInfoName());
-				if (urlDownloader->Terminate())
-				{
-					error("Terminated hanging download %s", urlDownloader->GetInfoName());
-					nzbInfo->SetUrlStatus(NzbInfo::lsNone);
-				}
-				else
-				{
-					error("Could not terminate hanging download %s", urlDownloader->GetInfoName());
-				}
-
-				// it's not safe to destroy urlDownloader, because the state of object is unknown
-				delete urlDownloader;
-
-				return true;
-			}
-			return false;
-		}),
-		m_activeDownloads.end());
+			error("Cancelling hanging url download %s", urlDownloader->GetInfoName());
+			urlDownloader->Stop();
+		}
+	}
 }
 
 void UrlCoordinator::LogDebugInfo()
@@ -212,8 +226,6 @@ void UrlCoordinator::LogDebugInfo()
  */
 NzbInfo* UrlCoordinator::GetNextUrl(DownloadQueue* downloadQueue)
 {
-	bool pauseDownload = g_Options->GetPauseDownload();
-
 	NzbInfo* nzbInfo = nullptr;
 
 	for (NzbInfo* nzbInfo1 : downloadQueue->GetQueue())
@@ -221,7 +233,6 @@ NzbInfo* UrlCoordinator::GetNextUrl(DownloadQueue* downloadQueue)
 		if (nzbInfo1->GetKind() == NzbInfo::nkUrl &&
 			nzbInfo1->GetUrlStatus() == NzbInfo::lsNone &&
 			nzbInfo1->GetDeleteStatus() == NzbInfo::dsNone &&
-			(!pauseDownload || g_Options->GetUrlForce()) &&
 			(!nzbInfo || nzbInfo1->GetPriority() > nzbInfo->GetPriority()))
 		{
 			nzbInfo = nzbInfo1;
@@ -295,13 +306,11 @@ void UrlCoordinator::UrlCompleted(UrlDownloader* urlDownloader)
 		// remove downloader from downloader list
 		m_activeDownloads.erase(std::find(m_activeDownloads.begin(), m_activeDownloads.end(), urlDownloader));
 
-		nzbInfo->SetActiveDownloads(0);
-
 		retry = urlDownloader->GetStatus() == WebDownloader::adRetry && !nzbInfo->GetDeleting();
 
 		if (nzbInfo->GetDeleting())
 		{
-			nzbInfo->SetDeleteStatus(NzbInfo::dsManual);
+			nzbInfo->SetDeleteStatus(nzbInfo->GetDeleteStatus() == NzbInfo::dsNone ? NzbInfo::dsManual : nzbInfo->GetDeleteStatus());
 			nzbInfo->SetUrlStatus(NzbInfo::lsNone);
 			nzbInfo->SetDeleting(false);
 		}
@@ -327,6 +336,7 @@ void UrlCoordinator::UrlCompleted(UrlDownloader* urlDownloader)
 
 	if (retry)
 	{
+		nzbInfo->SetActiveDownloads(0);
 		return;
 	}
 
@@ -342,7 +352,7 @@ void UrlCoordinator::UrlCompleted(UrlDownloader* urlDownloader)
 
 		if (addStatus == Scanner::asSuccess)
 		{
-			// if scanner has successfully added nzb-file to queue, our pNZBInfo is
+			// if scanner has successfully added nzb-file to queue, our nzbInfo is
 			// already removed from queue and destroyed
 			return;
 		}
@@ -354,31 +364,13 @@ void UrlCoordinator::UrlCompleted(UrlDownloader* urlDownloader)
 
 	g_QueueScriptCoordinator->EnqueueScript(nzbInfo, QueueScriptCoordinator::qeUrlCompleted);
 
-	std::unique_ptr<NzbInfo> oldNzbInfo;
-
 	{
 		GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 
-		// delete URL from queue
-		oldNzbInfo = downloadQueue->GetQueue()->Remove(nzbInfo);
+		nzbInfo->SetActiveDownloads(0);
 
-		// add failed URL to history
-		if (g_Options->GetKeepHistory() > 0 &&
-			nzbInfo->GetUrlStatus() != NzbInfo::lsFinished &&
-			!nzbInfo->GetAvoidHistory())
-		{
-			std::unique_ptr<HistoryInfo> historyInfo = std::make_unique<HistoryInfo>(std::move(oldNzbInfo));
-			historyInfo->SetTime(Util::CurrentTime());
-			downloadQueue->GetHistory()->Add(std::move(historyInfo), true);
-			downloadQueue->HistoryChanged();
-		}
-
-		downloadQueue->Save();
-	}
-
-	if (oldNzbInfo)
-	{
-		g_DiskState->DiscardFiles(oldNzbInfo.get());
+		DownloadQueue::Aspect aspect = {DownloadQueue::eaUrlFailed, downloadQueue, nzbInfo, nullptr};
+		downloadQueue->Notify(&aspect);
 	}
 }
 
@@ -398,26 +390,40 @@ bool UrlCoordinator::DeleteQueueEntry(DownloadQueue* downloadQueue, NzbInfo* nzb
 				return true;
 			}
 		}
+
+		return false;
 	}
 
 	info("Deleting URL %s", nzbInfo->GetName());
 
-	nzbInfo->SetDeleteStatus(NzbInfo::dsManual);
+	nzbInfo->SetDeleteStatus(nzbInfo->GetDeleteStatus() == NzbInfo::dsNone ? NzbInfo::dsManual : nzbInfo->GetDeleteStatus());
 	nzbInfo->SetUrlStatus(NzbInfo::lsNone);
 
-	std::unique_ptr<NzbInfo> oldNzbInfo = downloadQueue->GetQueue()->Remove(nzbInfo);
-
-	if (g_Options->GetKeepHistory() > 0 && !avoidHistory)
-	{
-		std::unique_ptr<HistoryInfo> historyInfo = std::make_unique<HistoryInfo>(std::move(oldNzbInfo));
-		historyInfo->SetTime(Util::CurrentTime());
-		downloadQueue->GetHistory()->Add(std::move(historyInfo), true);
-		downloadQueue->HistoryChanged();
-	}
-	else
-	{
-		g_DiskState->DiscardFiles(oldNzbInfo.get());
-	}
+	DownloadQueue::Aspect deletedAspect = {DownloadQueue::eaUrlDeleted, downloadQueue, nzbInfo, nullptr};
+	downloadQueue->Notify(&deletedAspect);
 
 	return true;
 }
+
+void UrlCoordinator::AddUrlToQueue(std::unique_ptr<NzbInfo> nzbInfo, bool addFirst)
+{
+	debug("Adding URL to queue");
+										
+	NzbInfo* addedNzb = nzbInfo.get();
+
+	GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
+
+	DownloadQueue::Aspect foundAspect = {DownloadQueue::eaUrlFound, downloadQueue, addedNzb, nullptr};
+	downloadQueue->Notify(&foundAspect);
+
+	if (addedNzb->GetDeleteStatus() != NzbInfo::dsManual)
+	{
+		downloadQueue->GetQueue()->Add(std::move(nzbInfo), addFirst);
+
+		DownloadQueue::Aspect addedAspect = {DownloadQueue::eaUrlAdded, downloadQueue, addedNzb, nullptr};
+		downloadQueue->Notify(&addedAspect);
+	}
+
+	downloadQueue->Save();
+}
+
