@@ -1,5 +1,5 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@sourceforge.net>
  *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
@@ -79,7 +79,7 @@ void RemoteServer::Run()
 				break;
 			}
 			m_connection.reset();
-			usleep(500 * 1000);
+			Util::Sleep(500);
 			continue;
 		}
 
@@ -112,7 +112,7 @@ void RemoteServer::Run()
 			Guard guard(m_processorsMutex);
 			completed = m_activeProcessors.size() == 0;
 		}
-		usleep(100 * 1000);
+		Util::Sleep(100);
 	}
 	debug("RemoteServer: request processor are completed");
 
@@ -125,6 +125,7 @@ void RemoteServer::Stop()
 	if (m_connection)
 	{
 		m_connection->SetSuppressErrors(true);
+		m_connection->SetForceClose(true);
 		m_connection->Cancel();
 
 		debug("Stopping RequestProcessors");
@@ -138,6 +139,18 @@ void RemoteServer::Stop()
 	debug("RemoteServer stop end");
 }
 
+void RemoteServer::ForceStop()
+{
+	debug("Killing RequestProcessors");
+	Guard guard(m_processorsMutex);
+	for (RequestProcessor* requestProcessor : m_activeProcessors)
+	{
+		requestProcessor->Kill();
+	}
+	m_activeProcessors.clear();
+	debug("RequestProcessors are killed");
+}
+
 void RemoteServer::Update(Subject* caller, void* aspect)
 {
 	debug("Notification from RequestProcessor received");
@@ -164,6 +177,9 @@ void RequestProcessor::Run()
 void RequestProcessor::Stop()
 {
 	Thread::Stop();
+#ifdef WIN32
+	m_connection->SetForceClose(true);
+#endif
 	m_connection->Cancel();
 }
 
