@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,13 +15,15 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #ifndef UTIL_H
 #define UTIL_H
 
+#include <optional>
+#include <type_traits>
 #include "NString.h"
 
 #ifdef WIN32
@@ -33,7 +36,11 @@ class Util
 {
 public:
 	static bool MatchFileExt(const char* filename, const char* extensionList, const char* listSeparator);
-	static int64 GetCurrentTicks();
+	static std::optional<std::string> 
+	FindExecutorProgram(const std::string& filename, const std::string& customPath);
+	static std::optional<std::string> 
+	FindShellOverriddenExecutor(const std::string& fileExt, const std::string& customPath);
+	static std::optional<std::string> FindPython();
 
 	/*
 	* Split command line into arguments.
@@ -44,35 +51,141 @@ public:
 
 	static int64 JoinInt64(uint32 Hi, uint32 Lo);
 	static void SplitInt64(int64 Int64, uint32* Hi, uint32* Lo);
+	static std::optional<double> StrToNum(const std::string& str);
 
 	static void TrimRight(char* str);
+	static void TrimRight(std::string& str);
+	static void TrimLeft(std::string& str);
 	static char* Trim(char* str);
+	static void Trim(std::string& str);
 	static bool EmptyStr(const char* str) { return !str || !*str; }
 	static std::vector<CString> SplitStr(const char* str, const char* separators);
 	static bool EndsWith(const char* str, const char* suffix, bool caseSensitive);
 	static bool AlphaNum(const char* str);
 
+	template<typename T, typename U>
+	static constexpr bool CmpEqual(T t, U u) noexcept
+	{
+		if constexpr (std::is_signed_v<T> == std::is_signed_v<U>)
+		{
+			return t == u;
+		}
+		else if constexpr (std::is_signed_v<T>)
+		{
+			return t >= 0 && std::make_unsigned_t<T>(t) == u;
+		}
+		else
+		{
+			return u >= 0 && std::make_unsigned_t<U>(u) == t;
+		}
+	}
+	
+	template<typename T, typename U>
+	static constexpr bool CmpNotEqual(T t, U u) noexcept
+	{
+		return !CmpEqual(t, u);
+	}
+	
+	template<typename T, typename U>
+	static constexpr bool CmpLess(T t, U u) noexcept
+	{
+		if constexpr (std::is_signed_v<T> == std::is_signed_v<U>)
+		{
+			return t < u;
+		}
+		else if constexpr (std::is_signed_v<T>)
+		{
+			return t < 0 || std::make_unsigned_t<T>(t) < u;
+		}
+		else
+		{
+			return u >= 0 && t < std::make_unsigned_t<U>(u);
+		}
+	}
+	
+	template<typename T, typename U>
+	static constexpr bool CmpGreater(T t, U u) noexcept
+	{
+		return CmpLess(u, t);
+	}
+	
+	template<typename T, typename U>
+	static constexpr bool CmpLessEqual(T t, U u) noexcept
+	{
+		return !CmpLess(u, t);
+	}
+	
+	template<typename T, typename U>
+	static constexpr bool CmpGreaterEqual(T t, U u) noexcept
+	{
+		return !CmpLess(t, u);
+	}
+	
+	template <typename From, typename To,
+	 	typename std::enable_if_t<std::is_integral_v<From> && std::is_integral_v<To>, bool> = true>
+	static constexpr To SafeIntCast(From num) noexcept
+	{
+		if constexpr (std::is_unsigned_v<From> && std::is_signed_v<To>)
+		{
+			if (CmpGreater(num, std::numeric_limits<To>::max()))
+			{
+				return 0;
+			}
+
+			return static_cast<To>(num);
+		}
+		else if constexpr (std::is_signed_v<From> && std::is_unsigned_v<To>)
+		{
+			if (num < 0)
+			{
+				return 0;
+			}
+
+			return static_cast<To>(num);
+		}
+		else if (CmpGreater(num, std::numeric_limits<To>::max()) || CmpLess(num, std::numeric_limits<To>::min()))
+		{
+			return 0;
+		}
+		else
+		{
+			return static_cast<To>(num);
+		}
+	}
+	
+
 	/* replace all occurences of szFrom to szTo in string szStr with a limitation that szTo must be shorter than szFrom */
 	static char* ReduceStr(char* str, const char* from, const char* to);
 
 	/* Calculate Hash using Bob Jenkins (1996) algorithm */
 	static uint32 HashBJ96(const char* buffer, int bufSize, uint32 initValue);
 
+	static std::unique_ptr<FILE, std::function<void(FILE*)>> MakePipe(const std::string& cmd);
+
+	static const char* NULL_OUTPUT;
+	static const char* FIND_CMD;
+
 #ifdef WIN32
 	static bool RegReadStr(HKEY keyRoot, const char* keyName, const char* valueName, char* buffer, int* bufLen);
+#else
+	static std::optional<std::string> Uname(const char* key);
 #endif
 
 	static void SetStandByMode(bool standBy);
 
 	static time_t CurrentTime();
+	static int64 CurrentTicks();
+	static void Sleep(int milliseconds);
 
 	/* cross platform version of GNU timegm, which is similar to mktime but takes an UTC time as parameter */
 	static time_t Timegm(tm const *t);
 
+	static bool StrCaseCmp(const std::string& a, const std::string& b);
+
 	static void FormatTime(time_t timeSec, char* buffer, int bufsize);
 	static CString FormatTime(time_t timeSec);
 
-	static CString FormatSpeed(int bytesPerSecond);
+	static CString FormatSpeed(int64 bytesPerSecond);
 	static CString FormatSize(int64 fileSize);
 	static CString FormatBuffer(const char* buf, int len);
 
@@ -86,10 +199,6 @@ public:
 
 	static void Init();
 
-	static uint32 Crc32(uchar *block, uint32 length);
-	static uint32 Crc32m(uint32 startCrc, uchar *block, uint32 length);
-	static uint32 Crc32Combine(uint32 crc1, uint32 crc2, uint32 len2);
-
 	/*
 	* Returns number of available CPU cores or -1 if it could not be determined
 	*/
@@ -294,7 +403,7 @@ public:
 	EStatus Read(const void **outputBuffer, int *outputBufferLength);
 
 private:
-	z_stream m_zStream = {0};
+	z_stream m_zStream{};
 	std::unique_ptr<Bytef[]> m_outputBuffer;
 	int m_bufferSize;
 	bool m_active = false;
@@ -317,4 +426,24 @@ private:
 	bool m_working = false;
 };
 
+class Crc32
+{
+public:
+	Crc32() { Reset(); }
+	void Reset();
+	void Append(uchar* block, uint32 length);
+	uint32 Finish();
+	static uint32 Combine(uint32 crc1, uint32 crc2, uint32 len2);
+
+private:
+#if defined(WIN32) && !defined(_WIN64)
+	// VC++ in 32 bit mode can not "alignas(16)" dynamically allocated objects
+	alignas(8) uint32_t m_state[4 * 5 + 8]; // = YEncode::crc_state
+	void* State() { void* p = &m_state; size_t s = sizeof(m_state); return std::align(16, 4 * 5, p, s); }
+#else
+	alignas(16) uint32_t m_state[4 * 5]; // = YEncode::crc_state
+	void* State() { return &m_state; }
+#endif
+};
+
 #endif
