@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2015-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,64 +15,75 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #include "nzbget.h"
 
-#include "catch.h"
+#include <boost/test/unit_test.hpp>
 
 #include "Options.h"
 #include "ParChecker.h"
 #include "TestUtil.h"
+#include "FileSystem.h"
+#include "YEncode.h"
 
-class ParCheckerMock: public ParChecker
+static std::string currDir = FileSystem::GetCurrentDirectory().Str();
+static std::string testDataDir = currDir + PATH_SEPARATOR + "parchecker";
+
+class ParCheckerMock : public ParChecker
 {
 public:
-	ParCheckerMock();
+	ParCheckerMock(std::string workingDir);
 	void Execute();
 	void CorruptFile(const char* filename, int offset);
+	~ParCheckerMock()
+	{
+		CString errmsg;
+		FileSystem::DeleteDirectoryWithContent(m_workingDir.c_str(), errmsg);
+	}
 
 protected:
-	virtual bool RequestMorePars(int blockNeeded, int* blockFound) { return false; }
-	virtual EFileStatus FindFileCrc(const char* filename, uint32* crc, SegmentList* segments);
+	bool RequestMorePars(int blockNeeded, int* blockFound) override { return false; }
+	EFileStatus FindFileCrc(const char* filename, uint32* crc, SegmentList* segments) override;
 
 private:
 	uint32 CalcFileCrc(const char* filename);
+	std::string m_workingDir;
 };
 
-ParCheckerMock::ParCheckerMock()
+ParCheckerMock::ParCheckerMock(std::string workingDir) : m_workingDir(std::move(workingDir))
 {
-	TestUtil::PrepareWorkingDir("parchecker");
-	SetDestDir(TestUtil::WorkingDir().c_str());
+	SetDestDir(m_workingDir.c_str());
+	BOOST_REQUIRE(FileSystem::CreateDirectory(m_workingDir.c_str()));
+	TestUtil::CopyAllFiles(m_workingDir.c_str(), testDataDir.c_str());
 }
 
 void ParCheckerMock::Execute()
 {
-	TestUtil::DisableCout();
 	ParChecker::Execute();
-	TestUtil::EnableCout();
 }
 
 void ParCheckerMock::CorruptFile(const char* filename, int offset)
 {
-	std::string fullfilename(TestUtil::WorkingDir() + "/" + filename);
+	std::string fullfilename(m_workingDir + PATH_SEPARATOR + filename);
 
 	FILE* file = fopen(fullfilename.c_str(), FOPEN_RBP);
-	REQUIRE(file != nullptr);
+	BOOST_REQUIRE(file != nullptr);
 
 	fseek(file, offset, SEEK_SET);
 	char b = 0;
 	int written = fwrite(&b, 1, 1, file);
-	REQUIRE(written == 1);
+	BOOST_REQUIRE(written == 1);
 
 	fclose(file);
 }
 
 ParCheckerMock::EFileStatus ParCheckerMock::FindFileCrc(const char* filename, uint32* crc, SegmentList* segments)
 {
-	std::ifstream sm((TestUtil::WorkingDir() + "/crc.txt").c_str());
+	std::string crcFileName = m_workingDir + PATH_SEPARATOR + "crc.txt";
+	std::ifstream sm(crcFileName);
 	std::string smfilename, smcrc;
 	while (!sm.eof())
 	{
@@ -79,7 +91,7 @@ ParCheckerMock::EFileStatus ParCheckerMock::FindFileCrc(const char* filename, ui
 		if (smfilename == filename)
 		{
 			*crc = strtoul(smcrc.c_str(), nullptr, 16);
-			uint32 realCrc = CalcFileCrc((TestUtil::WorkingDir() + "/" + filename).c_str());
+			uint32 realCrc = CalcFileCrc((m_workingDir + PATH_SEPARATOR + filename).c_str());
 			return *crc == realCrc ? ParChecker::fsSuccess : ParChecker::fsUnknown;
 		}
 	}
@@ -89,75 +101,71 @@ ParCheckerMock::EFileStatus ParCheckerMock::FindFileCrc(const char* filename, ui
 uint32 ParCheckerMock::CalcFileCrc(const char* filename)
 {
 	FILE* infile = fopen(filename, FOPEN_RB);
-	REQUIRE(infile);
+	BOOST_REQUIRE(infile);
 
 	CharBuffer buffer(1024 * 64);
-	uint32 downloadCrc = 0xFFFFFFFF;
+	Crc32 downloadCrc;
 
 	int cnt = buffer.Size();
 	while (cnt == buffer.Size())
 	{
 		cnt = (int)fread(buffer, 1, buffer.Size(), infile);
-		downloadCrc = Util::Crc32m(downloadCrc, (uchar*)(char*)buffer, cnt);
+		downloadCrc.Append((uchar*)(char*)buffer, cnt);
 	}
 
 	fclose(infile);
 
-	downloadCrc ^= 0xFFFFFFFF;
-	return downloadCrc;
+	return downloadCrc.Finish();
 }
 
-TEST_CASE("Par-checker: repair not needed", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(RepairNoNeedTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "RepairNoNeedTest");
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psRepairNotNeeded);
-	REQUIRE(parChecker.GetParFull() == true);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psRepairNotNeeded);
+	BOOST_CHECK(parChecker.GetParFull() == true);
 }
 
-TEST_CASE("Par-checker: repair possible", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(RepairPossibleTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=no");
-	cmdOpts.push_back("BrokenLog=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "RepairPossibleTest");
 	parChecker.CorruptFile("testfile.dat", 20000);
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psRepairPossible);
-	REQUIRE(parChecker.GetParFull() == true);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psRepairPossible);
+	BOOST_CHECK(parChecker.GetParFull() == true);
 }
 
-TEST_CASE("Par-checker: repair successful", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(RepairSuccessfulTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=yes");
-	cmdOpts.push_back("BrokenLog=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "RepairSuccessfulTest");
 	parChecker.CorruptFile("testfile.dat", 20000);
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psRepaired);
-	REQUIRE(parChecker.GetParFull() == true);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psRepaired);
+	BOOST_CHECK(parChecker.GetParFull() == true);
 }
 
-TEST_CASE("Par-checker: repair failed", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(RepairFailedTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=no");
-	cmdOpts.push_back("BrokenLog=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "RepairSuccessfulTest");
 	parChecker.CorruptFile("testfile.dat", 20000);
 	parChecker.CorruptFile("testfile.dat", 30000);
 	parChecker.CorruptFile("testfile.dat", 40000);
@@ -167,48 +175,52 @@ TEST_CASE("Par-checker: repair failed", "[Par][ParChecker][Slow][TestData]")
 	parChecker.CorruptFile("testfile.dat", 80000);
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psFailed);
-	REQUIRE(parChecker.GetParFull() == true);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psFailed);
+	BOOST_CHECK(parChecker.GetParFull() == true);
 }
 
-TEST_CASE("Par-checker: quick verification repair not needed", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(QuickVerificationRepairNotNeededTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	YEncode::init();
+
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "QuickVerificationRepairNotNeededTest");
 	parChecker.SetParQuick(true);
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psRepairNotNeeded);
-	REQUIRE(parChecker.GetParFull() == false);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psRepairNotNeeded);
+	BOOST_CHECK(parChecker.GetParFull() == false);
 }
 
-TEST_CASE("Par-checker: quick verification repair successful", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(QuickVerificationRepairSuccessfulTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=yes");
-	cmdOpts.push_back("BrokenLog=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	YEncode::init();
+
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "QuickVerificationRepairSuccessfulTest");
 	parChecker.SetParQuick(true);
 	parChecker.CorruptFile("testfile.dat", 20000);
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psRepaired);
-	REQUIRE(parChecker.GetParFull() == false);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psRepaired);
+	BOOST_CHECK(parChecker.GetParFull() == false);
 }
 
-TEST_CASE("Par-checker: quick full verification repair successful", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(QuickFullVerificationRepairSuccessfulTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=yes");
-	cmdOpts.push_back("BrokenLog=no");
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	YEncode::init();
+
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "QuickFullVerificationRepairSuccessfulTest");
 	parChecker.SetParQuick(true);
 	parChecker.CorruptFile("testfile.dat", 20000);
 	parChecker.CorruptFile("testfile.nfo", 100);
@@ -216,33 +228,23 @@ TEST_CASE("Par-checker: quick full verification repair successful", "[Par][ParCh
 
 	// All files were damaged, the full verification was performed
 
-	REQUIRE(parChecker.GetStatus() == ParChecker::psRepaired);
-	REQUIRE(parChecker.GetParFull() == true);
+	BOOST_CHECK(parChecker.GetStatus() == ParChecker::psRepaired);
+	BOOST_CHECK(parChecker.GetParFull() == true);
 }
 
-TEST_CASE("Par-checker: ignoring extensions", "[Par][ParChecker][Slow][TestData]")
+BOOST_AUTO_TEST_CASE(ParIgnoreExtDatTest)
 {
 	Options::CmdOptList cmdOpts;
 	cmdOpts.push_back("ParRepair=yes");
-	cmdOpts.push_back("BrokenLog=no");
 
 	ParChecker::EStatus expectedStatus;
 
-	SECTION("ParIgnoreExt")
-	{
-		cmdOpts.push_back("ParIgnoreExt=.dat");
-		expectedStatus = ParChecker::psRepairNotNeeded;
-	}
-
-	SECTION("ExtCleanupDisk")
-	{
-		cmdOpts.push_back("ExtCleanupDisk=.dat");
-		expectedStatus = ParChecker::psFailed;
-	}
+	cmdOpts.push_back("ExtCleanupDisk=.dat");
+	expectedStatus = ParChecker::psFailed;
 
 	Options options(&cmdOpts, nullptr);
 
-	ParCheckerMock parChecker;
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "ParIgnoreExtDatTest");
 	parChecker.CorruptFile("testfile.dat", 20000);
 	parChecker.CorruptFile("testfile.dat", 30000);
 	parChecker.CorruptFile("testfile.dat", 40000);
@@ -253,5 +255,31 @@ TEST_CASE("Par-checker: ignoring extensions", "[Par][ParChecker][Slow][TestData]
 
 	parChecker.Execute();
 
-	REQUIRE(parChecker.GetStatus() == expectedStatus);
+	BOOST_CHECK(parChecker.GetStatus() == expectedStatus);
+}
+
+BOOST_AUTO_TEST_CASE(ExtCleanupDiskDatTest)
+{
+	Options::CmdOptList cmdOpts;
+	cmdOpts.push_back("ParRepair=yes");
+
+	ParChecker::EStatus expectedStatus;
+
+	cmdOpts.push_back("ExtCleanupDisk=.dat");
+	expectedStatus = ParChecker::psFailed;
+
+	Options options(&cmdOpts, nullptr);
+
+	ParCheckerMock parChecker(currDir + PATH_SEPARATOR + "ExtCleanupDiskDatTest");
+	parChecker.CorruptFile("testfile.dat", 20000);
+	parChecker.CorruptFile("testfile.dat", 30000);
+	parChecker.CorruptFile("testfile.dat", 40000);
+	parChecker.CorruptFile("testfile.dat", 50000);
+	parChecker.CorruptFile("testfile.dat", 60000);
+	parChecker.CorruptFile("testfile.dat", 70000);
+	parChecker.CorruptFile("testfile.dat", 80000);
+
+	parChecker.Execute();
+
+	BOOST_CHECK(parChecker.GetStatus() == expectedStatus);
 }
