@@ -1,5 +1,5 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2015-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
@@ -20,10 +20,15 @@
 
 #include "nzbget.h"
 
-#include "catch.h"
+#include <iostream>
+#include <cassert>
 
+#include "Util.h"
 #include "FileSystem.h"
 #include "TestUtil.h"
+#include "Log.h"
+#include "Options.h"
+#include "DiskState.h"
 
 bool TestUtil::m_usedWorkingDir = false;
 std::string DataDir;
@@ -93,13 +98,13 @@ void TestUtil::PrepareWorkingDir(const std::string templateDir)
 	FileSystem::DeleteDirectoryWithContent(workDir.c_str(), errmsg);
 	while (FileSystem::DirectoryExists(workDir.c_str()) && retries > 0)
 	{
-		usleep(1000 * 100);
+		Util::Sleep(100);
 		retries--;
 		FileSystem::DeleteDirectoryWithContent(workDir.c_str(), errmsg);
 	}
-	REQUIRE_FALSE(FileSystem::DirectoryExists(workDir.c_str()));
+	assert(FileSystem::DirectoryExists(workDir.c_str()) == false);
 	FileSystem::CreateDirectory(workDir.c_str());
-	REQUIRE(FileSystem::DirEmpty(workDir.c_str()));
+	assert(FileSystem::DirEmpty(workDir.c_str()));
 
 	CopyAllFiles(workDir, srcDir);
 }
@@ -111,7 +116,7 @@ void TestUtil::CopyAllFiles(const std::string destDir, const std::string srcDir)
 	{
 		std::string srcFile(srcDir + "/" + filename);
 		std::string dstFile(destDir + "/" + filename);
-		REQUIRE(FileSystem::CopyFile(srcFile.c_str(), dstFile.c_str()));
+		FileSystem::CopyFile(srcFile.c_str(), dstFile.c_str());
 	}
 }
 
