@@ -1,8 +1,9 @@
 /*
- * This file is part of nzbget. See <http://nzbget.net>.
- *
- * Copyright (C) 2012-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * This file is part of nzbget. See <https://nzbget.com>.
  *
+ * Copyright (C) 2012-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * Copyright (C) 2024-2025 Denis <denis@nzbget.com>
+ * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -30,6 +31,33 @@ var Util = (new function($)
 {
 	'use strict';
 
+	this.emptyObject = function(obj) {
+		if (!obj) return true;
+		if (Object.keys(obj).length == 0) return true;
+		return false;
+	}
+
+	this.saveToLocalStorage = function(key, val)
+	{
+		try 
+		{
+			localStorage.setItem(key, val);
+		} catch (error) 
+		{
+			console.error(error);
+		}
+	}
+
+	this.getFromLocalStorage = function(key)
+	{
+		return localStorage.getItem(key);
+	}
+
+	this.removeFromLocalStorage = function(key)
+	{
+		return localStorage.removeItem(key);
+	}
+
 	this.formatTimeHMS = function(sec)
 	{
 		var hms = '';
@@ -57,7 +85,6 @@ var Util = (new function($)
 
 	this.formatTimeLeft = function(sec)
 	{
-		var hms = '';
 		var days = Math.floor(sec / 86400);
 		var hours = Math.floor((sec % 86400) / 3600);
 		var minutes = Math.floor((sec / 60) % 60);
@@ -83,8 +110,35 @@ var Util = (new function($)
 		return seconds + 's';
 	}
 
+	this.isNumber = function(value)
+	{
+		return typeof value === 'number';
+	}
+
+	this.getDaySinceUnixEpoch = function (date) 
+	{
+		var utcDate = new Date(Date.UTC(
+			date.getFullYear(),
+			date.getMonth(),
+			date.getDate()
+		));
+
+		// Get the number of milliseconds since the Unix epoch (UTC).
+		var utcTime = utcDate.getTime();
+
+		// Divide by the number of milliseconds in a day to get the number of days.
+		var daysSinceEpoch = Math.floor(utcTime / (1000 * 60 * 60 * 24));
+
+		return daysSinceEpoch;
+	}
+
 	this.formatDateTime = function(unixTime)
 	{
+		if (!Util.isNumber(unixTime) || unixTime < 0)
+		{
+			return '';
+		}
+
 		var dt = new Date(unixTime * 1000);
 		var h = dt.getHours();
 		var m = dt.getMinutes();
@@ -101,60 +155,117 @@ var Util = (new function($)
 
 		if (sizeMB >= 1024 * 1024 * 100)
 		{
-			return this.round0(sizeMB / 1024.0 / 1024.0) + '&nbsp;TB';
+			return Math.floor(sizeMB / 1024.0 / 1024.0) + ' TB';
 		}
 		else if (sizeMB >= 1024 * 1024 * 10)
 		{
-			return this.round1(sizeMB / 1024.0 / 1024.0) + '&nbsp;TB';
+			return this.round1(sizeMB / 1024.0 / 1024.0) + ' TB';
 		}
 		else if (sizeMB >= 1024 * 1000)
 		{
-			return this.round2(sizeMB / 1024.0 / 1024.0) + '&nbsp;TB';
+			return this.round2(sizeMB / 1024.0 / 1024.0) + ' TB';
 		}
 		else if (sizeMB >= 1024 * 100)
 		{
-			return this.round0(sizeMB / 1024.0) + '&nbsp;GB';
+			return Math.floor(sizeMB / 1024.0) + ' GB';
 		}
 		else if (sizeMB >= 1024 * 10)
 		{
-			return this.round1(sizeMB / 1024.0) + '&nbsp;GB';
+			return this.round1(sizeMB / 1024.0) + ' GB';
 		}
 		else if (sizeMB >= 1000)
 		{
-			return this.round2(sizeMB / 1024.0) + '&nbsp;GB';
+			return this.round2(sizeMB / 1024.0) + ' GB';
 		}
 		else if (sizeMB >= 100)
 		{
-			return this.round0(sizeMB) + '&nbsp;MB';
+			return Math.floor(sizeMB) + ' MB';
 		}
 		else if (sizeMB >= 10)
 		{
-			return this.round1(sizeMB) + '&nbsp;MB';
+			return this.round1(sizeMB) + ' MB';
 		}
 		else
 		{
-			return this.round2(sizeMB) + '&nbsp;MB';
+			return this.round2(sizeMB) + ' MB';
 		}
 	}
 
-	this.formatSpeed = function(bytesPerSec)
+	this.formatSpeed = function (bytesPerSec) 
 	{
-		if (bytesPerSec >= 100 * 1024 * 1024)
+		if (!Util.isNumber(bytesPerSec) || bytesPerSec < 0) 
 		{
-			return Util.round0(bytesPerSec / 1024.0 / 1024.0) + '&nbsp;MB/s';
+			return '';
 		}
-		else if (bytesPerSec >= 10 * 1024 * 1024)
+
+		if (bytesPerSec >= 100 * 1024 * 1024 * 1024) 
 		{
-			return Util.round1(bytesPerSec / 1024.0 / 1024.0) + '&nbsp;MB/s';
+			return Util.round0(bytesPerSec / 1024 / 1024 / 1024) + ' GB/s';
 		}
-		else if (bytesPerSec >= 1024 * 1000)
+		else if (bytesPerSec >= 10 * 1024 * 1024 * 1024) 
 		{
-			return Util.round2(bytesPerSec / 1024.0 / 1024.0) + '&nbsp;MB/s';
+			return Util.round1(bytesPerSec / 1024 / 1024 / 1024) + ' GB/s';
 		}
-		else
+		else if (bytesPerSec >= 1024 * 1024 * 1024) 
 		{
-			return Util.round0(bytesPerSec / 1024.0) + '&nbsp;KB/s';
+			return Util.round2(bytesPerSec / 1024 / 1024 / 1024) + ' GB/s';
 		}
+		if (bytesPerSec >= 100 * 1024 * 1024) 
+		{
+			return Util.round0(bytesPerSec / 1024.0 / 1024.0) + ' MB/s';
+		}
+		else if (bytesPerSec >= 10 * 1024 * 1024) 
+		{
+			return Util.round1(bytesPerSec / 1024.0 / 1024.0) + ' MB/s';
+		}
+		else if (bytesPerSec >= 1024 * 1000) 
+		{
+			return Util.round2(bytesPerSec / 1024.0 / 1024.0) + ' MB/s';
+		}
+
+		return Util.round0(bytesPerSec / 1024.0) + ' KB/s';
+	}
+
+	this.formatNetworkSpeed = function(speedMbps) 
+	{
+		if (!Util.isNumber(speedMbps))
+		{
+			return '';
+		}
+
+		if (!speedMbps || speedMbps < 0)
+		{
+			return '';
+		}
+
+		if (speedMbps >= 10000) 
+		{
+			return Util.round1(speedMbps / 10000) + ' Gbps';
+		}
+		if (speedMbps >= 1000) 
+		{
+			return Util.round1(speedMbps / 1000) + ' Gbps';
+		} 
+		else if (speedMbps >= 100) 
+		{
+			return Util.round0(speedMbps) + ' Mbps';
+		} 
+		else if (speedMbps >= 10) 
+		{
+			return Util.round1(speedMbps) + ' Mbps';
+		} 
+		else if (speedMbps >= 1) 
+		{
+			return Util.round2(speedMbps) + ' Mbps';
+		}
+
+		return Util.round0(speedMbps * 1000) + ' Kbps';
+	}
+
+	this.formatSpeedWithCustomUnit = function (bytesPerSec, unit) 
+	{
+		var res = this.formatSpeed(bytesPerSec);
+		return res.replace('B', unit);
 	}
 
 	this.formatAge = function(time)
@@ -179,6 +290,31 @@ var Util = (new function($)
 		}
 	}
 
+	this.formatNumber = function (num) 
+	{
+		var formattedNum;
+
+		if (num >= 1000000000) 
+		{
+			formattedNum = (num / 1000000000).toFixed(0);
+			return formattedNum + 'B';
+		}
+		else if (num >= 1000000) 
+		{
+			formattedNum = (num / 1000000).toFixed(0);
+			return formattedNum + 'M';
+		}
+		else if (num >= 1000) 
+		{
+			formattedNum = (num / 1000).toFixed(0);
+			return formattedNum + 'K';
+		}
+		else 
+		{
+			return num.toFixed(0);
+		}
+	}
+
 	this.round0 = function(arg)
 	{
 		return Math.round(arg);
@@ -342,6 +478,16 @@ var Util = (new function($)
 	    return text.substring(text.length - substr.length, text.length) === substr;
 	}
 
+	this.makeId = function(text)
+	{
+		return text.replace(/ |\/|\\|\.|\$|\:|\*/g, '_');
+	}
+
+	this.joinInt64 = function(hi, lo)
+	{
+		return (hi << 32) + lo;
+	}
+
 }(jQuery));
 
 
@@ -379,7 +525,6 @@ var TabDialog = (new function($)
 		var dialog = this;
 		var body = $('.modal-body', dialog);
 		var footer = $('.modal-footer', dialog);
-		var header = $('.modal-header', dialog);
 		dialog.css({margin: '', left: '', top: '', bottom: '', right: '', width: '', height: ''});
 		body.css({position: '', height: '', left: '', right: '', top: '', bottom: '', 'max-height': ''});
 		footer.css({position: '', left: '', right: '', bottom: ''});
@@ -400,7 +545,6 @@ var TabDialog = (new function($)
 		var header = $('.modal-header', dialog);
 
 		var oldBodyHeight = body.height();
-		var oldWinHeight = dialog.height();
 		var windowWidth = $(window).width();
 		var windowHeight = $(window).height();
 		var oldTabWidth = fromTab.width();
@@ -530,53 +674,95 @@ var RPC = (new function($)
 	'use strict';
 
 	// Properties
-	this.rpcUrl;
-	this.defaultFailureCallback;
+	this.rpcUrl = '';
+	this.defaultFailureCallback = undefined;
 	this.connectErrorMessage = 'Cannot establish connection';
+	this.safeMethods = [];
+	this.etags = {};
 
-	this.call = function(method, params, completed_callback, failure_callback, timeout, custom_headers)
+	this.openRequest = function(method, params, options)
 	{
-		var _this = this;
-
-		var request = JSON.stringify({nocache: new Date().getTime(), method: method, params: params});
 		var xhr = new XMLHttpRequest();
 
-		xhr.open('post', this.rpcUrl);
-
-		if (timeout)
+		if (this.safeMethods.indexOf(method) > -1)
 		{
-			xhr.timeout = timeout;
+			var request = '';
+			for (var i = 0; i < params.length; i++)
+			{
+				request += '&=' + encodeURIComponent(params[i]);
+			}
+			if (params.length > 0)
+			{
+				request = '?' + request.substr(1);
+			}
+			xhr.open('get', this.rpcUrl + '/' + method + request);
+			xhr._reportRequest = method + request;
+		}
+		else
+		{
+			xhr.open('post', this.rpcUrl);
+			xhr._request = JSON.stringify({nocache: new Date().getTime(), method: method, params: params});
+			xhr._reportRequest = xhr._request;
 		}
 
-		for (var i = 0; i < (custom_headers ? custom_headers.length : 0); i++)
+		if (options && ('timeout' in options))
 		{
-			xhr.setRequestHeader(custom_headers[i].name, custom_headers[i].value);
+			xhr.timeout = options['timeout'];
 		}
 
+		for (var i = 0; i < (options && ('custom_headers' in options) ? options['custom_headers'].length : 0); i++)
+		{
+			xhr.setRequestHeader(options['custom_headers'][i].name, options['custom_headers'][i].value);
+		}
+
+		return xhr;
+	}
+
+	this.call = function(method, params, completed_callback, failure_callback, options)
+	{
+		var _this = this;
+		var xhr = this.openRequest(method, params, options);
+
 		xhr.onreadystatechange = function()
 		{
 			if (xhr.readyState === 4)
 			{
 					var res = 'Unknown error';
 					var result;
+					var cached = false;
 					if (xhr.status === 200)
 					{
 						if (xhr.responseText != '')
 						{
-							try
+							var etag = xhr.getResponseHeader('ETag');
+							if (etag)
 							{
-								result = JSON.parse(xhr.responseText);
+								cached = RPC.etags[method] == etag;
+								RPC.etags[method] = etag;
 							}
-							catch (e)
+
+							if (cached && options && ('prefer_cached' in options) && options['prefer_cached'])
 							{
-								res = e;
+								result = {result: null, error: null};
 							}
+							else
+							{
+								try
+								{
+									result = JSON.parse(xhr.responseText);
+								}
+								catch (e)
+								{
+									res = e;
+								}
+							}
+
 							if (result)
 							{
 								if (result.error == null)
 								{
 									res = result.result;
-									completed_callback(res);
+									completed_callback(res, cached);
 									return;
 								}
 								else
@@ -584,7 +770,7 @@ var RPC = (new function($)
 									res = result.error.message;
 									if (result.error.message != 'Access denied')
 									{
-										res = res + '<br><br>Request: ' + request;
+										res = res + '<br><br>Request: ' + xhr._reportRequest;
 									}
 								}
 							}
@@ -613,6 +799,6 @@ var RPC = (new function($)
 					}
 			}
 		};
-		xhr.send(request);
+		xhr.send(xhr._request);
 	}
 }(jQuery));
